//
//  **************************************************************************
//
//  controller.cpp
//
//  (C) 2003 Bosco K. Ho 
//
//  
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#include "event.h"
#include "controller.h"
#include "pdb.h"


enum {
  CMD_NULL,
  CMD_GENERIC,
  CMD_PRO_PUCKER,
  CMD_PHI,
  CMD_PSI,
  CMD_CHI,
  CMD_LOAD,
  CMD_SAVE,
  CMD_DELETE,
  CMD_INSERT,
  CMD_LOOP,
  CMD_LOOP_DRIVE,
};



class Cmd
{
public:
  Cmd(wxFrame* pParent, MyProtein*  pProtein): 
    mpParent(pParent), mpProtein(pProtein), mKey(CMD_GENERIC)
    { mKey = CMD_NULL; }
  virtual ~Cmd() {}

  virtual void execute() {}
  virtual void undo() {}
  
  std::string name;
  wxFrame* mpParent;
  MyProtein*  mpProtein;
  int mKey;
};



class CopyProteinCmd : public Cmd
{
public:
  CopyProteinCmd(wxFrame* pParent, MyProtein* pProtein): 
    Cmd(pParent, pProtein), mProteinCopy(*mpProtein) { mKey = CMD_GENERIC; }
  
  void execute() {}
  void undo() 
  { 
    *mpProtein = mProteinCopy; 
    mpProtein->mSelect.reset();
  }
  
  MyProtein mProteinCopy;
};




class LoadCmd : public CopyProteinCmd
{
public:
  LoadCmd(wxFrame* pParent, MyProtein*  pProtein, std::string fname): 
    CopyProteinCmd(pParent, pProtein)
  { 
    mKey = CMD_LOAD;
    mFname = fname;
  }
  
  void execute() 
  {
    PdbReader pdb(mFname.c_str()); 
    pdb.loadChain(*mpProtein);
    updateName();
    mpProtein->mSelect.reset();
    mpProtein->mLoopClosure.clear();
    mpProtein->mMeasure.clear();
    mpProtein->countCovalentBonds();
    mpProtein->countInteractions();
    mpProtein->mSelect.reset();
  }
  void undo() 
  { 
    CopyProteinCmd::undo(); 
    updateName();
  }
    
  void updateName()
  {
    wxString text;
    text.Printf(_T("[%s] %s"), mpProtein->getPdbCode().c_str(), mpProtein->getName().c_str());
    mpParent->SetStatusText(text);
  }
  
  std::string mFname;
};



class LoopSolutionCmd : public Cmd
{
public:
  LoopSolutionCmd(wxFrame* pParent, MyProtein* pProtein, int i_soln): 
    Cmd(pParent, pProtein), miSoln(i_soln), 
    mOldRes1(mpProtein->mLoopClosure.res(0)),
    mOldRes2(mpProtein->mLoopClosure.res(1)),
    mOldRes3(mpProtein->mLoopClosure.res(2))
  { 
    mpProtein->extractFragment(mSavedLoop, mOldRes1, mOldRes3);
    mKey = CMD_LOOP;  
  }

  void execute() 
  {
    mpProtein->mLoopClosure.setSolution(miSoln);
    mpProtein->countInteractions();
    mpProtein->mIsChanged = true;
  }
  void undo() 
  { 
    mpProtein->mLoopClosure.copyLoopToProtein(&mSavedLoop, mOldRes1);
    mpProtein->extractAngles();
    mpProtein->countInteractions();
    mpProtein->mIsChanged = true;
  }
  int miSoln;
  int mOldRes1;
  int mOldRes2;
  int mOldRes3;
  Protein mSavedLoop;
};



class LoopDriveCmd : public LoopSolutionCmd
{
public:
  LoopDriveCmd(wxFrame* pParent, MyProtein* pProtein,
    int i, double phi, double psi): 
    LoopSolutionCmd(pParent, pProtein, -1),
    miRes(i), mPhi(phi), mPsi(psi)
  { 
     mKey = CMD_LOOP;  
  }

  void execute() 
  {
    mpProtein->mLoopClosure.drive(miRes, mPhi, mPsi);
    mpProtein->mIsChanged = true;
  }
  
  int miRes;
  double mPhi;
  double mPsi;
};



class SetPhiCmd: public Cmd
{
public:
  SetPhiCmd(wxFrame* pParent, MyProtein*  pProtein, int i, double phi):
    Cmd(pParent, pProtein), miRes(i), mPhi(phi)
    {
      mOldPhi = mpProtein->phi(miRes);
      normalize_angle(mPhi);
      mKey = CMD_PHI;
    }
  void execute() 
    { 
      mpProtein->setPhi(miRes, mPhi);
      mpProtein->mIsChanged = true;
    }
  void undo() 
    { 
      mpProtein->setPhi(miRes, mOldPhi); 
    }
   
  int miRes;
  double mOldPhi;
  double mPhi;
};



class SetProPuckerCmd: public Cmd
{
public:
  SetProPuckerCmd(wxFrame* pParent, MyProtein*  pProtein, int i, double pucker):
    Cmd(pParent, pProtein), miRes(i), mPucker(pucker)
    {
      mOldPucker = mpProtein->proPucker(miRes);
      mKey = CMD_PRO_PUCKER;
    }
  void execute() 
    { 
      mpProtein->setProPucker(miRes, mPucker);
      mpProtein->mIsChanged = true;
    }
  void undo() 
    { 
      mpProtein->setProPucker(miRes, mOldPucker); 
    }
   
  int miRes;
  double mOldPucker;
  double mPucker;
};



class SetPsiCmd : public Cmd
{
public:
  SetPsiCmd(wxFrame* pParent, MyProtein*  pProtein, int i, double psi):
    Cmd(pParent, pProtein), miRes(i), mPsi(psi)
    {
      mOldPsi = mpProtein->psi(miRes);
      normalize_angle(mPsi);
      mKey = CMD_PSI;
    }
  void execute() 
    { 
      mpProtein->setPsi(miRes, mPsi); 
      mpProtein->mIsChanged = true;
    }
  void undo() 
    { 
      mpProtein->setPsi(miRes, mOldPsi); 
    }
   
  int miRes;
  double mOldPsi;
  double mPsi;
};




class SetChiCmd : public Cmd
{
public:
  SetChiCmd(wxFrame* pParent, MyProtein*  pProtein, int i, int iChi, double chi):
    Cmd(pParent, pProtein), miChi(iChi), miRes(i), mChi(chi) 
    {
      mOldChi = mpProtein->chi(miRes, iChi);
      normalize_angle(mChi);
      mKey = CMD_CHI;
    }
  void execute() 
    { 
      mpProtein->setChi(miRes, miChi, mChi); 
      mpProtein->mIsChanged = true;
    }
  void undo() 
    { 
      mpProtein->setChi(miRes, miChi, mOldChi); 
    }
   
  int miChi;
  int miRes;
  double mChi;
  double mOldChi;
};




class DeleteCmd : public CopyProteinCmd
{
public:
  DeleteCmd(wxFrame* pParent, MyProtein*  pProtein): 
    CopyProteinCmd(pParent, pProtein)
    { 
      mKey = CMD_DELETE;
    }
  
  void execute() 
    {
      MyProtein& protein = *(mpProtein);
  
      int left = 0;
      int right = mpProtein->nRes()-1;
      
      if (protein.mSelect.nSelected() > 0)
      {
        mpProtein->mLoopClosure.clear();
        mpProtein->mMeasure.clear();
      }
      
      while (right >= 0)
      {
        if (protein.mSelect.isSelected(right))
        {
          left = right;
          while ((left-1) >= 0) 
            if (protein.mSelect.isSelected(left-1))
              left -= 1;
            else
              break;
       
          protein.eraseResAndSplice(left, right);

          if (protein.mFocus.res > right)
            protein.mFocus.res -= (right - left) + 1;
          else if ((protein.mFocus.res >= left) && (protein.mFocus.res <= right))
            protein.mFocus.res = left;
            
          if (protein.mFocus.res >= protein.nRes())
            protein.mFocus.res = protein.nRes()-1;
          
          if (!protein.hasAtom(protein.mFocus.res, protein.mFocus.atom))
            protein.mFocus.atom = CA;
                      
          right = left;
          
        }
        right -= 1;
      }

      // make sure there are no proteins with just blocking groups
      if (protein.nRes() == 2)
        if ( (ACE == protein.resKey(0)) && (NME == protein.resKey(1)) )
        {
          protein.eraseRes(0);
          protein.eraseRes(1);
        }

      mpProtein->countCovalentBonds();
      mpProtein->countInteractions();

      mpProtein->mSelect.reset();
      
      mpProtein->mIsChanged = true;
      mpParent->Refresh();

    }
};



class InsertCmd : public CopyProteinCmd
{
public:
  InsertCmd(wxFrame* pParent, MyProtein*  pProtein, 
            MyProtein* pClipboard, int i): 
    CopyProteinCmd(pParent, pProtein),
    mpClipboard(pClipboard), miRes(i)
  { 
    mKey = CMD_INSERT;
  }
  
  void execute() 
  {
		if (mpClipboard->isEmpty()) 
			return;

		if (mpProtein->isEmpty()) 
			miRes	=	0;

		if (ACE	== mpProtein->resKey(miRes))
			miRes++;

		if (miRes-1 >= 0)
		  if (NME	== mpProtein->resKey(miRes-1))
			  miRes--;

		if (0	== miRes)
			mpProtein->unblockNTerminal();
		
		if (mpProtein->nRes() ==	miRes)
			mpProtein->unblockCTerminal();
			
		mpProtein->insertFragmentAndSplice(*mpClipboard, miRes);
		
		mpProtein->mSelect.reset();
		for	(int j=miRes;	j<=mpClipboard->nRes()+miRes-1; j++)
			mpProtein->mSelect.set(j, true);

		if (mpProtein->mFocus.res	>= miRes)
			mpProtein->mFocus.res	+= mpClipboard->nRes();
    if (!mpProtein->hasAtom(mpProtein->mFocus.res, mpProtein->mFocus.atom))
      mpProtein->mFocus.atom = CA;

    mpProtein->mLoopClosure.clear();
    mpProtein->mMeasure.clear();
		mpProtein->countCovalentBonds();
    mpProtein->countInteractions();

    mpProtein->mIsChanged = true;

		mpParent->Refresh();
  }

  MyProtein* mpClipboard;
  int miRes;
};




class BuildHydrogenCmd : public CopyProteinCmd
{
public:
  BuildHydrogenCmd(wxFrame* pParent, MyProtein* pProtein): 
    CopyProteinCmd(pParent, pProtein)  { mKey = CMD_LOAD;  }
  void execute() 
    {
      for (int i=0; i<mpProtein->nRes(); i++)
        mpProtein->buildAllHydrogens(i);
      mpProtein->countCovalentBonds();
      mpProtein->countInteractions();
    }
};



class BlockTerminalsCmd : public CopyProteinCmd
{
public:
  BlockTerminalsCmd(wxFrame* pParent, MyProtein* pProtein): 
    CopyProteinCmd(pParent, pProtein)  { mKey = CMD_LOAD;  }
  void execute() 
    {
      if (mpProtein->resKey(0) != ACE)
      {
        mpProtein->mFocus.res += 1;
        for (int i=mpProtein->nRes()-1; i > 0; i--)
          mpProtein->mSelect.set(i, mpProtein->mSelect.isSelected(i-1));   
      }
      mpProtein->blockTerminals();
      mpProtein->countCovalentBonds();
      mpProtein->countInteractions();
      mpProtein->mIsChanged = true;
    }
};



class ChargeTerminalCmd : public CopyProteinCmd
{
public:
  ChargeTerminalCmd(wxFrame* pParent, MyProtein* pProtein): 
    CopyProteinCmd(pParent, pProtein)  { mKey = CMD_LOAD;  }
  void execute() 
    {
      if (mpProtein->resKey(0) == ACE)
      {
        mpProtein->mFocus.res -= 1;
        for (int i=0; i < mpProtein->nRes()-1; i++)
          mpProtein->mSelect.set(i, mpProtein->mSelect.isSelected(i+1));   
      }
      mpProtein->chargeTerminals();
      mpProtein->countCovalentBonds();
      mpProtein->countInteractions();
      mpProtein->mIsChanged = true;
    }
};




class MutateCmd : public CopyProteinCmd
{
public:
  MutateCmd(wxFrame* pParent, MyProtein* pProtein, int resKey): 
    CopyProteinCmd(pParent, pProtein),
    mResKey(resKey)
      { mKey = CMD_LOAD;  }
  void execute() 
    {
      for (int i=0; i<mpProtein->nRes(); i++)
        if (mpProtein->mSelect.isSelected(i))
        {
          mpProtein->mutateRes(i, mResKey);
        }
      while (!mpProtein->mFocus.isValid())
        mpProtein->mFocus.atom -= 1;
      mpProtein->countCovalentBonds();
      mpProtein->countInteractions();
      mpProtein->mIsChanged = true;
      mpParent->Refresh();
    }
  int mResKey;
};




//////////////////////////////////////////////
// The controller of the protein model
/////////////////////////////////////////////



void Controller::setMenuLabel(std::string name)
{
  wxMenuItem *item = mpParent->GetMenuBar()->FindItem(EVENT_UNDO);
  if (name == "")
    name = "empty";
  item->SetText(("&Undo " + name + "\tCtrl-Z").c_str());
}



void Controller::setUndoMarker(std::string name)
{ 
  Cmd* cmd = new Cmd(mpParent, mpProtein);
  cmd->name = name;
  process(cmd); 
  setMenuLabel(cmd->name);
}



void Controller::process(Cmd* pCmd) 
{ 
  mCmdList.push_back(pCmd);
  pCmd->execute();
}



void Controller::undo()
{
  Cmd* pCmd;

  // checks to see if the top command 
  // is a marker
  // and deltes if it is

  pCmd = mCmdList.back();
  if (pCmd->mKey == CMD_NULL)
  {
    delete pCmd;
    mCmdList.pop_back();
  }
  
  while (canUndo())
  {
    pCmd = mCmdList.back();

    if (pCmd->mKey == CMD_NULL)
    {
      setMenuLabel(pCmd->name);
      break;
    }
    else
    {
      pCmd->undo();
      delete pCmd;
      mCmdList.pop_back();
    }
  }

  if (!canUndo())
    setMenuLabel("");
  
  mpProtein->mLoopClosure.clear();
  mpProtein->mSelect.reset();
  mpProtein->mMeasure.clear();
  
  mpProtein->countCovalentBonds();
  mpProtein->countInteractions();
}



void Controller::setPhiPsi(double phi, double psi)
{
  for (int i=0; i<=mpProtein->nRes(); i++)
    if (mpProtein->mSelect.isSelected(i))
    {
      bool is_in_loop = false;
      bool is_a_hinge = false;

      if (mpProtein->mLoopClosure.nSelectedRes() == 3)
      {
        int low = mpProtein->mLoopClosure.res(0);
        int med = mpProtein->mLoopClosure.res(1);
        int hi  = mpProtein->mLoopClosure.res(2);
      
        if ((i > low) && (i < med))
          is_in_loop = true;
        else if ((i > med) && (i < hi))
          is_in_loop = true;

        if ((i==low) || (i==med) || (i==hi))
          is_a_hinge = true;
      }
    
      if (is_in_loop)
        process(new LoopDriveCmd(mpParent, mpProtein, i, phi, psi));          
      else if (!is_a_hinge)
      {
        process(new SetPhiCmd(mpParent, mpProtein, i, phi)); 
        process(new SetPsiCmd(mpParent, mpProtein, i, psi)); 
      }
    }
}



void Controller::setChi(int iChi, double chi)
{
  for (int i=0; i<=mpProtein->nRes(); i++)
    if (mpProtein->mSelect.isSelected(i))
    {
      int n_chi = get_n_chi(mpProtein->resKey(i));
      if (iChi < n_chi)
        process(new SetChiCmd(mpParent, mpProtein, i, iChi, chi)); 
    }
}



void Controller::setProPucker(double pucker)
{
  int count = 0;
  for (int i=0; i<=mpProtein->nRes(); i++)
    if (mpProtein->mSelect.isSelected(i))
      if (mpProtein->resKey(i) == PRO)
      {
        if (pucker >= 0.0)
          pucker = 1.0;
        else
          pucker = -1.0;
        if (pucker*mpProtein->proPucker(i) < 0.0)
        {
          count += 1;
          process(new SetProPuckerCmd(mpParent, mpProtein, i, pucker)); 
        }
    }
}



void Controller::loadProtein(std::string fname)
{
  resetSelect();
  clearLoopClosure();
  clearMeasure();
  process(new LoadCmd(mpParent, mpProtein, fname)); 
  setUndoMarker("load PDB file");
}



void Controller::saveProtein(std::string fname)
{
  PdbWriter pdb(fname.c_str()); 
  pdb.writeChain(*mpProtein, true);
} 



void Controller::insertFromPdb(std::string fname)
{
  PdbReader pdb(fname.c_str());
  pdb.loadChain(mClipboardProtein);
  process(new InsertCmd(mpParent, mpProtein, &mClipboardProtein, mpProtein->mInsertRes));
  setUndoMarker("load PDB file");
}



void Controller::copyToPdb(std::string fname)
{
  copySelectionToClipboard();
  PdbWriter pdb(fname.c_str()); 
  pdb.writeChain(mClipboardProtein, true);
} 



void Controller::selectAll()
{
  for (int i=0; i<mpProtein->nRes(); i++)
    setSelect(i, true);
} 



void Controller::copySelectionToClipboard()
{
  MyProtein& protein = *(mpProtein);
  if (protein.mSelect.nSelected() == 0)
    return;
  
  mClipboardProtein.clear();
  
  int left=0;
  int right;
  while (left<protein.nRes())
  {
    if (protein.mSelect.isSelected(left))
    {
      right=left;
      while (protein.mSelect.isSelected(right+1) && ((right+1)<protein.nRes()))
        right++;
   
      Protein temp_frag;
      protein.extractFragment(temp_frag, left, right);

      temp_frag.unblockNTerminal();
      temp_frag.unblockNTerminal();
      
      mClipboardProtein.insertFragmentAndSplice(temp_frag, mClipboardProtein.nRes());
      left = right;
    }
    left++;
  }
}



void Controller::deleteSelection()
{
  process(new DeleteCmd(mpParent, mpProtein));
  setUndoMarker("delete");
}



void Controller::insertClipboardInProtein(int i)
{
  process(new InsertCmd(mpParent, mpProtein, &mClipboardProtein, i));
  setUndoMarker("insert");
}



void Controller::insertAlanine()
{
  mClipboardProtein.clear();
  mClipboardProtein.appendRes(ALA);
  mClipboardProtein.setPhi(0, -150.0);
  mClipboardProtein.setPsi(0, 120.0);
  process(new InsertCmd(mpParent, mpProtein, &mClipboardProtein, mpProtein->mInsertRes));
  setUndoMarker("insert");
}



void Controller::makeAlaninePeptide()
{
  make_alanine_peptide(&mClipboardProtein);
  selectAll();
  process(new DeleteCmd(mpParent, mpProtein));
  process(new InsertCmd(mpParent, mpProtein, &mClipboardProtein, 0));
  resetSelect();
  setUndoMarker("make alanine peptide");
}



void Controller::makeAlphabetProtein()
{
  make_alphabet_protein(&mClipboardProtein);
  selectAll();
  process(new DeleteCmd(mpParent, mpProtein));
  process(new InsertCmd(mpParent, mpProtein, &mClipboardProtein, 0));
  resetSelect();
  setUndoMarker("make alphabet protein");
}



void Controller::chargeTerminals()
{
  process(new ChargeTerminalCmd(mpParent, mpProtein));
  setUndoMarker("charge terminal");
}



void Controller::buildHydrogen()
{
  process(new BuildHydrogenCmd(mpParent, mpProtein));
  setUndoMarker("build hydrogens");
}



void Controller::blockTerminals()
{
  process(new BlockTerminalsCmd(mpParent, mpProtein));
  setUndoMarker("block terminal");
}



void Controller::mutate(int resKey)
{
  process(new MutateCmd(mpParent, mpProtein, resKey));
  setUndoMarker("mutate residue(s)");
}



void Controller::setLoopSolution(int i_soln)
{
  if (i_soln != mpProtein->mLoopClosure.nSolution())
  {
    process(new LoopSolutionCmd(mpParent, mpProtein, i_soln));
    setUndoMarker("Discrete loop solution");
  }
}

