//
//  **************************************************************************
//
//  controls.cpp
//  (C) 2003 Bosco K. Ho 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//


#include "widget.h"
#include "protview.h"
#include "controls.h"
#include "dialog.h"
#include "event.h"
#include <vector>
#include <algorithm>


void RamaPlot::onLeftMouseDown()
{
  mpController->clearInteractions();
  onLeftMouseDrag();
  mIsLeftMouseDown = true;
}



void RamaPlot::onLeftMouseUp()
{
  mIsLeftMouseDown = false;
  mpController->countInteractions();
  mpController->setUndoMarker("backbone torsion");
  Refresh();
}



void RamaPlot::onLeftMouseDrag()
{
  double fraction_x = (mMouseX-getX()) / double(getWidth()-1);
  double fraction_y = ((getHeight()-1)-(mMouseY-getY())) / double(getHeight()-1);
  
  double phi = 360.0*fraction_x - 180.0;
  double psi = 360.0*fraction_y - 180.0;

  normalize_angle(phi);
  normalize_angle(psi);
  
  mpController->setPhiPsi(phi, psi);
 
  Refresh();
}



void RamaPlot::onRightMouseDown()
{
  TorsionDialog dialog(mpParent, mpController);
  dialog.ShowModal();
}



int RamaPlot::convertPhiToX(double phi)
{
  return (int)((phi+180)/360.0 * (getWidth()-1) + getX());
}



int RamaPlot::convertPsiToY(double psi)
{
  return (int)((-psi+180)/360.0 * (getHeight()-1) + getY());
}



void RamaPlot::drawSlantedEllipse(wxBufferedPaintDC &dc, int x1, int y1, int x2, int y2, double width)
{
  int x_c = (x1 + x2) / 2;
  int y_c = (y1 + y2) / 2;
  
  double dy = x2 - x1;
  double dx = -y2 + y1;

  double d = sqrt(dx*dx + dy*dy);
  
  double d_new = width * d / 2.0;
  
  dx = dx / d;
  dy = dy / d;

  int x3 = x_c + (int)(dx*d_new);
  int y3 = y_c + (int)(dy*d_new);
  
  int x4 = x_c - (int)(dx*d_new);
  int y4 = y_c - (int)(dy*d_new);
  
  dy = y2 - y1;
  dx = x2 - x1;

  dx = dx / d;
  dy = dy / d;

  double d_tangent = width * d / 3.0;

  wxPoint points[5];

  points[0] = wxPoint(x3, y3);
  points[1] = wxPoint(x3 - (int)(dx*d_tangent), y3 - (int)(dy*d_tangent));
  points[2] = wxPoint(x1, y1);
  points[3] = wxPoint(x4 - (int)(dx*d_tangent), y4 - (int)(dy*d_tangent));
  points[4] = wxPoint(x4, y4);
  dc.DrawSpline(5, points);

  points[0] = wxPoint(x3, y3);
  points[1] = wxPoint(x3 + (int)(dx*d_tangent), y3 + (int)(dy * d_tangent));
  points[2] = wxPoint(x2, y2);
  points[3] = wxPoint(x4 + (int)(dx * d_tangent), y4 + (int)(dy * d_tangent));
  points[4] = wxPoint(x4, y4);
  dc.DrawSpline(5, points);

}




void RamaPlot::draw(wxBufferedPaintDC &dc)
{
  MyProtein* p_protein = mpController->mpProtein;

  int x1 = getX();
  int x2 = getX() + getWidth() - 1;

  int y1 = getY();
  int y2 = getY() + getHeight() - 1;

  wxColor color;

  // draw Ramachandran-plot box
  if (mIsLeftMouseDown || isInside(mMouseX, mMouseY))
  {
    // highlight if mouse is hovering 
    color = wxColor("GREEN");
    dc.SetPen(wxPen(color, 2, wxSOLID));
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle(x1, y1, getWidth(), getHeight());

    if (isInside(mMouseX, mMouseY))
    { 
      dc.SetPen(wxPen(color, 1, wxSOLID));
      dc.DrawLine(mMouseX, y1, mMouseX, y2);
      dc.DrawLine(x1, mMouseY, x2, mMouseY);
    }
  }
  else
  {
    // plain box otherwise
    color = wxColor("LIGHT GREY");
    dc.SetPen(wxPen(color, 1, wxSOLID));
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle(x1, y1, getWidth(), getHeight());
  }

  // draw x-y axis
  int half;
  half = getWidth()/2;
  color = wxColor("LIGHT GREY");
  dc.SetPen(wxPen(color, 1, wxSOLID));
  dc.DrawLine(x1 + half, y1, x1 + half, y2);
  half = mHeight/2;
  dc.DrawLine(x1, y1 + half, x2, y1 + half);
  color = wxColor("LIGHT GREY");
  dc.SetTextForeground(color);

  // draw Ramachandran plot regions
  drawSlantedEllipse(dc, convertPhiToX(-175), convertPsiToY(175),
                         convertPhiToX(-90), convertPsiToY(110), 0.4);
  
  drawSlantedEllipse(dc, convertPhiToX(-90), convertPsiToY(175),
                        convertPhiToX(-50), convertPsiToY(110), 0.4);

  drawSlantedEllipse(dc, convertPhiToX(-80), convertPsiToY(110),
                        convertPhiToX(-80), convertPsiToY(60), 0.4);

  drawSlantedEllipse(dc, convertPhiToX(-120), convertPsiToY(100),
                        convertPhiToX(-120), convertPsiToY(70), 0.4);

  drawSlantedEllipse(dc, convertPhiToX(-120), convertPsiToY(30),
                         convertPhiToX(-40), convertPsiToY(-60), 0.4);

  drawSlantedEllipse(dc, convertPhiToX(50), convertPsiToY(70),
                         convertPhiToX(70), convertPsiToY(0), 0.3);
  

  // draw labels 
  dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));
  dc.SetBackgroundMode(wxTRANSPARENT);
  dc.SetTextForeground(color);

  // draw phi/psi labels
  wxString text;
  long length, height, descent;
  text = _T("phi");
  dc.GetTextExtent(text, &length, &height, &descent);
  dc.DrawText( text, x1 + 4, y2 - height*2 - 4);    
  text = _T("psi");
  dc.DrawText(text, x1 + 4, y2 - height - 4);    

  for (int i=0; i<=p_protein->nRes(); i++)
    if (p_protein->mSelect.isSelected(i))
    {
      // skip blocking groups
      if ( (p_protein->resKey(i) == ACE) || 
           (p_protein->resKey(i) == NME) )
         continue;
  
      // draw residue labels
      dc.SetTextForeground(wxColor("GREEN"));
      text.Printf(_T("%s-%d"), p_protein->resName(i).c_str(), 
                  p_protein->resNo(i));
      dc.GetTextExtent(text, &length, &height, &descent);
      dc.DrawText(text, x1 + 4, y2 - height*3 - 4);    

      // draw phi/psi values and marker
      double phi = (double)p_protein->phi(i);
      double psi = (double)p_protein->psi(i);
  
      dc.SetTextForeground(wxColor("LIGHT GREY"));

      text.Printf(_T("%7.f"), phi);
      dc.GetTextExtent(text, &length, &height, &descent);
      dc.DrawText(text, x2 - length - 4, y2 - 2*height - 4);    
  
      text.Printf(_T("%7.f"), psi);
      dc.GetTextExtent( text, &length, &height, &descent );
      dc.DrawText( text, x2 - length - 4, y2 - height - 4);    
  
      // draw marker
      int rX = (int)( ( phi+180)/360.0 * (getWidth()-1) + x1 );
      int rY = (int)( (-psi+180)/360.0 * (getHeight()-1) + y1 );
  
      dc.SetPen(wxPen(*wxBLACK, 1, wxSOLID));
  
      dc.SetBrush(wxColor("GREEN"));
      dc.DrawCircle(rX, rY, 5);

      if (p_protein->mLoopClosure.isSelectedRes(i))
      {
         dc.SetBrush(wxColor("PURPLE"));
         dc.DrawCircle(rX, rY, 3);
      } 

    }
}



//////////////////////////////////////////////////////
/// Chi angle control box
//////////////////////////////////////////////////////


int RamaPlotSizer::getX()
{ 
  return mpRamaPlot->getX() + mpRamaPlot->getWidth() + 2; 
}



int RamaPlotSizer::getY()
{ 
  return mpRamaPlot->getY(); 
}


  
void RamaPlotSizer::onLeftMouseDrag()
{
  mIsLeftMouseDown = true;

  int new_width = mMouseX - mpRamaPlot->getX() - 2;
  int new_height = mMouseY - mpRamaPlot->getY();

  int new_size;
  if (new_width > new_height)
    new_size = new_width;
  else
    new_size = new_height;
    
  if (new_size < 75)
    new_size = 75;

  mpRamaPlot->mHeight = new_size;
  mpRamaPlot->mWidth = new_size;

  mX = mpRamaPlot->getX() + new_size;
  mY = mpRamaPlot->getY() + new_size;
}



void RamaPlotSizer::onLeftMouseDown()
{
  onLeftMouseDrag();
}



void RamaPlotSizer::onLeftMouseUp()
{
  mIsLeftMouseDown = false;
}


void RamaPlotSizer::draw(wxBufferedPaintDC &dc)
{
  wxColor color;
  if (isInside(mMouseX, mMouseY) || mIsLeftMouseDown)
    dc.SetPen(wxPen(wxColor("GREEN"), 2, wxSOLID));
  else 
    dc.SetPen(wxPen(wxColor("LIGHT GREY"), 1, wxSOLID));
  dc.SetBrush(*wxTRANSPARENT_BRUSH);

  int size = getWidth();
  dc.DrawRectangle(getX(), getY(), size, size);

  dc.SetPen(wxPen(wxColor("LIGHT GREY"), 1, wxSOLID));
  for (int i=3; i<size-2; i+=3)
    dc.DrawLine(getX()+i, getY()+2, getX() + size - 3, getY() + size - i - 1);
}
  
  


//////////////////////////////////////////////////////
/// Chi angle control box
//////////////////////////////////////////////////////



bool ChiBox::isOn()
{
  return (mpRamaPlot->isOn());
}



void ChiBox::onLeftMouseDown()
{
  miChi = -1;
  mpController->clearInteractions();
  onLeftMouseDrag();
}



void ChiBox::onLeftMouseUp()
{
  miChi = -1;
  mpController->countInteractions();
  mpController->setUndoMarker("sidechain torsion");
  Refresh();
}



void ChiBox::onLeftMouseDrag()
{
  if (miChi == -1)
    miChi = convertYToIChi(mMouseY);

  double chi = convertXToChi(mMouseX);
  normalize_angle(chi);

  if (miChi == mnChiMax-1)
    mpController->setProPucker(chi);
  else
    mpController->setChi(miChi, chi);
  
  Refresh();
}



void ChiBox::onRightMouseDown()
{
  //TorsionDialog dialog(mpParent, mpController);
  //dialog.ShowModal();
}



int ChiBox::convertIChiToY(int miChi)
{
  return (int)( getY() + getHeight()*((mnChiMax-miChi-1)/double(mnChiMax)) );
}



int ChiBox::convertYToIChi(int y)
{
  return (int)( mnChiMax - (y-getY())/double(getHeight())*mnChiMax );
}



double ChiBox::convertXToChi(int x)
{
  return (x-getX())/double(getWidth())*360.0 + -180.0;
}



int ChiBox::convertChiToX(double chi)
{
  return ((int)((chi+180.0)/360.0*getWidth()) + getX());
}



void ChiBox::draw(wxBufferedPaintDC &dc)
{
  MyProtein* p_protein = mpController->mpProtein;

  int boxHeight = getHeight()/(mnChiMax);
  wxColor color;
  
  // draw boxes
  int x = getX();
  int y = getY();

  // draw whole box and dividers
  color = wxColor("LIGHT GREY");
  int half = getWidth()/2;
  dc.SetPen(wxPen(color, 1, wxSOLID));
  dc.DrawLine(x + half, y, x + half, y + getHeight());  
  dc.SetBrush(*wxTRANSPARENT_BRUSH);
  dc.DrawRectangle(x, y, getWidth(), getHeight());

  // draw box of each chi angle
  for (int n=0; n<mnChiMax; n++)
  {
    int y = convertIChiToY(n);
    
    // draw lower border of box for this chi
    color = wxColor("LIGHT GREY");
    dc.SetPen(wxPen(color, 1, wxSOLID));
    dc.DrawLine(x, y, x + getWidth(), y);
  }
  
  
  // draw chi value labels 
  for (int n=0; n<(mnChiMax-1); n++)
  {
    int y = convertIChiToY(n);
    
    dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));
    dc.SetBackgroundMode(wxTRANSPARENT);
    dc.SetTextForeground(color);
  
    wxString text;
    long length, height, descent;
    text.Printf(_T("chi%d"), n+1);
    dc.GetTextExtent(text, &length, &height, &descent);
    dc.DrawText( text, x + 4, y + boxHeight/2 - height/2);    
  }

  // identify the chi to highlight
  int i_chi = -1;
  if (miChi >= 0)
    i_chi = miChi;
  else if (isInside(mMouseX, mMouseY))
    i_chi = convertYToIChi(mMouseY);

  // draw highlight for chi
  if ((i_chi >= 0) && (i_chi < mnChiMax))
  {   
    // draw highlight box
    color = wxColor("GREEN");
    dc.SetPen(wxPen(color, 2, wxSOLID));
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    int y = convertIChiToY(i_chi);
    dc.DrawRectangle(x, y, getWidth(), boxHeight);
  
    // draw highlight cross-bar
    if (isInside(mMouseX, mMouseY))
    {
      dc.SetPen(wxPen(color, 1, wxSOLID));
      dc.DrawLine(mMouseX, y, mMouseX, y + boxHeight - 1);
    }
  }

  // now draw all the data points of the chi selected residues
  for (int i=0; i<p_protein->nRes(); i++)
    if (p_protein->mSelect.isSelected(i))
    {
      int n_chi = get_n_chi(p_protein->resKey(i));
      for (int j = 0; j<n_chi; j++)
      {
        double chi = p_protein->chi(i, j);
    
        int x = convertChiToX(chi);
        int y = convertIChiToY(j);

        // draw chi value
        wxString text;
        int length, height, descent;
        text.Printf(_T("%7.f"), chi);
        dc.GetTextExtent(text, &length, &height, &descent);
        dc.DrawText(text, getX() + getWidth() - length - 4 - 1, y + boxHeight/2 - height/2);    
      
        // draw chi marker
        dc.SetPen(wxPen(*wxBLACK, 1, wxSOLID));
        dc.SetBrush(wxColor("GREEN"));
        dc.DrawCircle(x, y + boxHeight/2, 5);
      }
      
      // handle Proline pucker
      if ((p_protein->resKey(i) == PRO))
      {
        int y = convertIChiToY(n_chi);

        wxString text;
        wxColor color;
        int length, height, descent;

        // draw DOWN pucker
        if (p_protein->proPucker(i) < 0.0)
          color = wxColor("GREEN");
        else
          color = wxColor("LIGHT GREY"); 
        dc.SetTextForeground(color);
        text.Printf(_T("DOWN"));
        dc.GetTextExtent(text, &length, &height, &descent);
        dc.DrawText(text, getX() + 4,  y + boxHeight/2 - height/2);
      
        // draw UP pucker
        if (p_protein->proPucker(i) > 0.0)
          color = wxColor("GREEN");
        else
          color = wxColor("LIGHT GREY"); 
        dc.SetTextForeground(color);
        text.Printf(_T("UP"));
        dc.GetTextExtent(text, &length, &height, &descent);
        dc.DrawText(text, getX() + getWidth() - 1 - length - 4,  y + boxHeight/2 - height/2);
      }
    }
}



//////////////////////////////////////////////////////
//  Sequence 
//////////////////////////////////////////////////////



void Sequence::shiftBounds(int i)
{
  int n_res = mpProtein->nRes();
  if ((i >= 0) && (i < n_res))
  {
    if (mnDisplayRes < n_res)
    {
      if (i < mLeftRes)
      {
         mLeftRes = i;
         mRightRes = i + mnDisplayRes - 1;
      }
      if (i > mRightRes)
      {
         mRightRes = i;
         mLeftRes = i - mnDisplayRes + 1;
      }
    }
  }
}  



void Sequence::onLeftMouseDClick()
{
  int i = convertXToRes(mMouseX);
  int n_res = mpProtein->nRes();

  if ((i >= 0) && (i < n_res))
  {
    mpController->setFocus(AtomMarker(mpProtein, i, CA));
    mpController->setSelect(i, true);
    Refresh();
  }
}


void Sequence::onLeftMouseDown()
{
  //mpController->mSelect.reset();
  
  int i = convertXToRes(mMouseX);
  int n_res = mpProtein->nRes();

  if ((i >= 0) && (i < n_res))
  {
    mDragSelect = !(mpProtein->mSelect.isSelected(i));
    mFixedLoopRes = i;
    mpController->setSelect(i, mDragSelect);
    Refresh();
  }
}



void Sequence::onLeftMouseDrag()
{
  int i = convertXToRes(mMouseX);
  int n_res = mpProtein->nRes();
  
  if ((i >= 0) && (i < n_res))
  {
    shiftBounds(i);
 
    //mpController->mSelect.reset();
  
    if (i<mFixedLoopRes)
    {
      for (int j=i; j<=mFixedLoopRes; j++)
        mpController->setSelect(j, mDragSelect);
    }
    else
    {
      for (int j=mFixedLoopRes; j<=i; j++)
        mpController->setSelect(j, mDragSelect);
    }

    Refresh();
  }
}



void Sequence::onRightMouseDown()
{
  int i = convertXToRes(mMouseX);
  if ((i < 0) || (i >= mpController->mpProtein->nRes()))
    return;
  mpController->setInsertRes(i);
  mpController->setSelect(i, true);
  Refresh();
  PopupMenu popupMenu(mpParent, mpController);
  popupMenu.show(mMouseX, mMouseY);
  Refresh();

}
 


void Sequence::checkBounds()
{
  int n_res = mpProtein->nRes();
  int last_res = n_res - 1;

  wxSize sz = GetClientSize();  
  mnDisplayResMax = (sz.GetWidth()-40)/mResWidth;

  if (n_res <= mnDisplayResMax)
  {
    mLeftRes = 0;
    mRightRes = last_res;
    mnDisplayRes = n_res;
  }
  else
  {
    mnDisplayRes = mnDisplayResMax;

    while (mRightRes > last_res) 
      mRightRes--;
      
    mLeftRes = mRightRes - mnDisplayRes + 1;

    while (mLeftRes < 0) 
    { 
      mLeftRes++;
      mRightRes++;
    }
  }
  
  mWidth = mResWidth * mnDisplayRes;
}



int Sequence::convertXToRes(int x)
{
  return (int)floor((x-getX())/(double)mResWidth) + mLeftRes;
}



int Sequence::convertResToX(int i)
{
  return (i-mLeftRes)*mResWidth + getX();
}



void Sequence::draw(wxBufferedPaintDC& dc)
{
  checkBounds();
  
  int hover_res = -1;
  if (isInside(mMouseX, mMouseY))
    hover_res = convertXToRes(mMouseX);
    
  // draw the sequence box
  for (int i = mLeftRes; i <= mRightRes; i+=1)
  {
    int x = convertResToX(i);
    int y = getY();
    
    // draw boxes of selected residues
    if (mpProtein->mSelect.isSelected(i)) 
    {
      dc.SetPen(wxPen("GREEN"));
      dc.SetBrush(wxBrush("GREEN"));
      dc.DrawRectangle(x, y, mResWidth, mResHeight);
    }
    
    // draw hover box for mouse
    if (hover_res == i)
    {
      dc.SetBrush(wxBrush(*wxTRANSPARENT_BRUSH));
      dc.SetPen(wxPen(wxColor("GREEN"), 1, wxSOLID));
      dc.DrawRectangle(x-1, y-1, mResWidth+2, mResHeight+2);
      dc.SetPen(wxPen("BLACK"));
      dc.DrawRectangle(x, y, mResWidth, mResHeight);
    }
    
    // draw the amino acid letters
    wxString text;
    int length, height, descent;

    dc.SetFont(wxFont(14, wxMODERN, wxNORMAL, wxNORMAL));
    dc.SetBackgroundMode(wxTRANSPARENT);

    if (mpProtein->mSelect.isSelected(i)) 
      dc.SetTextForeground("BLACK");
    else
      dc.SetTextForeground("GREEN");

    text = _T(get_res_char(mpProtein->resKey(i)));
    dc.GetTextExtent(text, &length, &height, &descent);
    dc.DrawText(text, x + mResWidth/2 - length/2, y + mResHeight/2 - height/2);

    // draw secondary structure 
    double phi = mpProtein->phi(i);
    double psi = mpProtein->psi(i);

    if ((phi > -180) && (phi < 0) && (psi > 60) && (psi < 180))
      text = _T("e");
    else if ((phi > -90) && (phi < 30) && (psi > -90) && (psi < 30))
      text = _T("h");
    else if ((phi > 0) && (phi < 90) && (psi > 0) && (psi < 90))
      text = _T("l");
    else
      text = _T("");

    dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));
    dc.SetTextForeground("LIGHT GREY");
    dc.GetTextExtent(text, &length, &height, &descent);
    dc.DrawText(text, x + mResWidth/2 - length/2, y + mResHeight - 2);
  }
}



//////////////////////////////////////////////////
// FocusPicker
//////////////////////////////////////////////////

int FocusPicker::getX()
{
  return mpSequence->getX();
}



int FocusPicker::getWidth()
{
  return mpSequence->getWidth();
}



void FocusPicker::onLeftMouseDown()
{
  int i = mpSequence->convertXToRes(mMouseX);
  int n_res = mpSequence->mpProtein->nRes();

  if ((i >= 0) && (i < n_res))
  {
    mpController->setFocus(AtomMarker(mpProtView->mpProtein, i, CA));
    Refresh();
  }
}



void FocusPicker::onLeftMouseDrag()
{
  onLeftMouseDown();  

  int i = mpSequence->convertXToRes(mMouseX);

  mpSequence->shiftBounds(i);
  Refresh();
}



void FocusPicker::draw(wxBufferedPaintDC& dc)
{
  
  int hover_res = -1;

  if (isInside(mMouseX, mMouseY))
    hover_res = mpSequence->convertXToRes(mMouseX);
    
  int res_width = mpSequence->mResWidth;
    
  // draw the sequence box
  for (int i = mpSequence->mLeftRes; i <= mpSequence->mRightRes; i+=1)
  {
    int x = mpSequence->convertResToX(i);
    int y = getY();

    dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));
    dc.SetBackgroundMode(wxTRANSPARENT);
    wxString text;

    int resNo = mpSequence->mpProtein->resNo(i);
    text.Printf(_T("%d"), resNo);

    int length, height, descent;
    dc.GetTextExtent(text, &length, &height, &descent);

    int x_half = x + res_width/2;
    int y_half = y + getHeight()/2;
    
    int x_char = x_half - length/2 - 1;
    int y_char = y_half - height/2 - 1;

    int x_offset = -2;
    int y_offset = -1;
    
    if (hover_res == i)
    {
      dc.SetPen(wxPen("PURPLE"));
      dc.SetBrush(wxBrush(*wxTRANSPARENT_BRUSH));
      dc.DrawRectangle(x_char+ x_offset, y_char + y_offset, length-2*x_offset, getHeight() - 2*y_offset);

      dc.SetTextForeground("LIGHT GREY");
      dc.DrawText(text, x_char, y_char);
    }
    
    // draw focus res
    if (mpProtView->mpProtein->mFocus.res == i)
    {
      dc.SetPen(wxPen("PURPLE"));
      dc.SetBrush(wxBrush("PURPLE"));
      dc.DrawRectangle(x_char+ x_offset, y_char + y_offset, length-2*x_offset, getHeight() -2*y_offset);

      dc.SetTextForeground("BLACK");
      dc.DrawText(text, x_char, y_char);
    }
    else
    {
      // draw numbered ticks
      int resNo = mpSequence->mpProtein->resNo(i);
      if (0 == (resNo % 5)) 
      {
        dc.SetTextForeground("LIGHT GREY");
        dc.DrawText(text, x_char, y_char);
      }
    }
  }
}



//////////////////////////////////////////////////
// Scrollbar object
//////////////////////////////////////////////////

int Scrollbar::convertXtoRes(int x)
{
  int n_res = mpSequence->mpProtein->nRes();
  return (int)( ((x - getX())/double(getWidth())) * n_res);
}



int Scrollbar::convertResToX(int i)
{
  int n_res = mpSequence->mpProtein->nRes();
  return (int)(i/double(n_res)*(getWidth())) + getX();
}



void Scrollbar::onLeftMouseDown()
{
  int& right_res = mpSequence->mRightRes;
  int& left_res = mpSequence->mLeftRes;

  int i = convertXtoRes(mMouseX);
  if ((i>=left_res) && (i<=right_res))
    mOffset = (left_res + right_res)/2 - i;
  else
    mOffset = 0;
  onLeftMouseDrag();
}



void Scrollbar::onLeftMouseDrag()
{
  int& right_res = mpSequence->mRightRes;
  int& left_res = mpSequence->mLeftRes;
  int n_res = mpSequence->mpProtein->nRes();
  int n_display_res = mpSequence->mnDisplayRes;
  
  if (n_res <= n_display_res)
    return;
    
  int new_center_res = convertXtoRes(mMouseX);
  int old_center_res = (right_res + left_res)/2;

  int disp = new_center_res - old_center_res;
  left_res += disp + mOffset;
  right_res += disp + mOffset;

  while (left_res < 0)
  {
    left_res++;
    right_res++;
  }

  while (right_res >= n_res)
  {
    right_res--;
    left_res--;
  }

  Refresh();
}

  

void Scrollbar::draw(wxBufferedPaintDC &dc)
{
  int& right_res = mpSequence->mRightRes;
  int& left_res = mpSequence->mLeftRes;
  int n_res = mpSequence->mpProtein->nRes();
  int n_display_res_max = mpSequence->mnDisplayResMax;
  
  if (n_res <= n_display_res_max) 
    return;

  int x1 = getX();
  int x2 = getX() + getWidth() - 1;
  int y1 = getY();
  int y2 = getY() + mHeight;
  int y_offset, x_offset;

  // draw bar highlight
  if (isInside(mMouseX, mMouseY))
  {
    dc.SetPen(wxColor("DARK GRAY"));
    dc.SetBrush(wxBrush("DARK GRAY"));
    y_offset = 4;
    x_offset = 2;
    dc.DrawRectangle(x1-x_offset, y1, getWidth()+2*x_offset, mHeight);
  }
    
  // draw selected residues
  int res_width = convertResToX(1) - convertResToX(0) + 1;
  y_offset = 1;
  dc.SetPen(wxColor("GREEN"));
  dc.SetBrush(wxBrush("GREEN"));
  for (int i=0; i < n_res; i++)
    if (mpSequence->mpProtein->mSelect.isSelected(i))
      dc.DrawRectangle(convertResToX(i), y1+y_offset, res_width, mHeight-2*y_offset);

  // draw focus residue
  y_offset = 1;
  dc.SetPen(wxColor("PURPLE"));
  dc.SetBrush(wxBrush("PURPLE"));  
  int foucs_res = mpProtView->mpProtein->mFocus.res;
  dc.DrawRectangle(convertResToX(foucs_res), y1+y_offset, res_width, mHeight-2*y_offset);

  // draw thread
  x_offset = -2;
  int y_half = (y1 + y2)/2;
  dc.SetPen(wxColor("BLACK"));
  //dc.DrawRectangle(x1+x_offset, y_half-1, mWidth-2*x_offset, 3);
  dc.SetPen(wxColor("LIGHT GREY"));
  dc.DrawLine(x1+x_offset, y_half, x2-x_offset, y_half);

  // draw ends
  x_offset = -2;
  y_offset = 0;
  dc.DrawLine(x1+x_offset, y1+y_offset, x1+x_offset, y2-y_offset);
  dc.DrawLine(x2-x_offset, y1+y_offset, x2-x_offset, y2-y_offset);

  // draw bar
  int bar_width = convertResToX(right_res) - convertResToX(left_res) + res_width - 1;
  int bar_x1 = convertResToX(left_res);

  dc.SetPen(wxColor("BLACK"));
  y_offset = 2;
  dc.DrawLine(bar_x1, y1+y_offset, bar_x1+bar_width-1, y1+y_offset);
  dc.DrawLine(bar_x1, y2-y_offset, bar_x1+bar_width-1, y2-y_offset);

  y_offset = 3;
  dc.SetPen(wxColor("LIGHT GREY"));
  dc.SetBrush(wxBrush("LIGHT GREY"));
  dc.DrawRectangle(bar_x1, y1+y_offset, bar_width, mHeight-2*y_offset);
  
}


//////////////////////////////////////////////////
// SlabBar object
//////////////////////////////////////////////////


void SlabBar::onLeftMouseDown()
{
  onLeftMouseDrag();
}



void SlabBar::onLeftMouseDrag()
{
  int new_bar_height = getY() + getHeight() - mMouseY;
  if (new_bar_height > getHeight())
    new_bar_height = getHeight();
  if (new_bar_height < 0)
    new_bar_height = 0;
  mpProtView->mSlabThickness = new_bar_height / float(getHeight()) * mMaxSlabThickness;
  if (mpProtView->mSlabThickness > (0.9 * mMaxSlabThickness))
    mpProtView->mIsSlab = false;
  else 
    mpProtView->mIsSlab = true;
  Refresh();
}

  
int mid_point(int x1, int width)
{
  if ((width % 2) == 0)
    return x1 + width/2 - 1;
  else
    return x1 + width/2;
}



void SlabBar::draw(wxBufferedPaintDC &dc)
{
  int x1 = getX();
  int x2 = getX() + getWidth() - 1;
  int y1 = getY();
  int y2 = getY() + getHeight() - 1;
 
  // draw label
  wxString text = _T("Z");
  int length, height, descent;
  dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));
  dc.SetTextForeground("LIGHT GREY");
  dc.GetTextExtent(text, &length, &height, &descent);
  dc.DrawText(text, x1, y1 - height - 3);

  // draw bar
  double slab_thickness = mpProtView->mSlabThickness;
  if (slab_thickness > 0.9 * mMaxSlabThickness)
    slab_thickness = mMaxSlabThickness;
    
  int bar_height =  (int)(getHeight()*(slab_thickness)/mMaxSlabThickness);
  int bar_y = getY() + getHeight() - bar_height;

  dc.SetPen(wxColor("LIGHT GREY"));
  dc.SetBrush(wxBrush("LIGHT GREY"));
  for (int y=bar_y; y<=y2; y+=2)
    dc.DrawLine(x1, y, x2, y);

  // draw box
  if (isInside(mMouseX, mMouseY))
    dc.SetPen(wxPen(wxColor("GREEN"), 2, wxSOLID));
  else 
    dc.SetPen(wxPen(wxColor("LIGHT GREY"), 1, wxSOLID));
  dc.SetBrush(*wxTRANSPARENT_BRUSH);
  dc.DrawRectangle(x1, y1, getWidth(), getHeight());

  dc.SetPen(wxPen(wxColor("LIGHT GREY"), 1, wxSOLID));
  dc.SetBrush(wxBrush("LIGHT GREY"));
  dc.DrawRectangle(x1 + 2, y1+ 2, getWidth() - 4, (int)(0.1*getHeight()) - 4);
    
}


/////////////////////////////
// Loop Solutions
/////////////////////////////

int LoopSolutions::getWidth()
{
  return mpController->mpProtein->mLoopClosure.nSolution() * getHeight(); 
}


int LoopSolutions::convertXToSoln(int x)
{
  return (x - getX()) / getHeight();
}


int LoopSolutions::convertSolnToX(int n)
{
  return n * getHeight() + getX();
}


void LoopSolutions::onLeftMouseDown()
{
  onLeftMouseDrag();
}


void LoopSolutions::onLeftMouseDrag()
{
  int i = convertXToSoln(mMouseX);
  if ((i>=0) && (i < mpController->mpProtein->mLoopClosure.nSolution()))
  {
    mpController->setLoopSolution(i);
    Refresh();
  }
}


void LoopSolutions::draw(wxBufferedPaintDC &dc)
{
  int curr = mpController->mpProtein->mLoopClosure.iSolution();
  int n = mpController->mpProtein->mLoopClosure.nSolution();
  int highlight = convertXToSoln(mMouseX);

  for (int i=0; i< n; i++)
  {
    int x = convertSolnToX(i);

    if (i==curr)
    {
      dc.SetPen(wxColor("PURPLE"));
      dc.SetBrush(wxBrush("PURPLE"));
    }
    else
    {
      dc.SetPen(wxColor("LIGHT GREY"));
      dc.SetBrush(wxBrush("LIGHT GREY"));
    }
    dc.DrawRectangle(x + 2, getY() + 2, getHeight() - 4, getHeight()-4);
    
    if (i==highlight)
    {  
      dc.SetPen(wxColor("PURPLE"));
      dc.SetBrush(*wxTRANSPARENT_BRUSH);
      dc.DrawRectangle(x, getY(), getHeight(), getHeight());
    }
    
  }    
}


//////////////////////////////////////////////

PickControl::PickControl(wxWindow* pParent, 
              ProtView* pProtView, int x, int y, int height):
              Widget(pParent, x, y, -1, height), 
              mpProtView(pProtView),
              mpController(pProtView->mpController)
              
{
  labels.push_back("focus");
  labels.push_back(" ");
  labels.push_back("select");
  labels.push_back("x");
  labels.push_back(" ");
  labels.push_back("measure");
  labels.push_back("x");
  labels.push_back(" ");
  labels.push_back("3-point-clamp");
  labels.push_back("x");

}



void PickControl::onLeftMouseDown()
{
  int choice = -1;
  
  int left_x = getX();
  int right_x;
  for (int i=0; i < (int)lengths.size(); i++)
  {
    right_x = left_x + lengths[i];
    if ((mMouseX >= left_x) && (mMouseX < right_x))
    {
      choice = i;
      break;
    }
    left_x = right_x;
  }
  
  switch (choice)
  {
    case 0: mpProtView->mPickState = PICK_FOCUS; break;
    case 2: mpProtView->mPickState = PICK_RESIDUE;  break;
    case 3: mpController->resetSelect(); break;
    case 5: mpProtView->mPickState = PICK_MEASURE; break;
    case 6: mpController->clearMeasure(); break;
    case 8: mpProtView->mPickState = PICK_ANCHOR; break;
    case 9: mpController->clearLoopClosure(); break;
  }
  
  Refresh();
}
    


void PickControl::draw(wxBufferedPaintDC &dc)
{
  
  if ((int)lengths.size() == 0)
  {
    // initialize
    wxString text;
    int length, height, descent;
    dc.SetFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL));

    mWidth = 0;
    for (int i=0; i < (int)labels.size(); i++)
    {
      text = _T(labels[i].c_str());
      dc.GetTextExtent(text, &length, &height, &descent);
      lengths.push_back(length + 4);
      mWidth += lengths[i];
    }
  }    
    
  int left_x = getX();
  int right_x;
  wxString text;
  for (int i=0; i < (int)labels.size(); i++)
  {    
    right_x = left_x + lengths[i];

    bool is_selected_label = false;

    if ((mpProtView->mPickState == PICK_FOCUS) && (i == 0))
      is_selected_label = true;
    if ((mpProtView->mPickState == PICK_RESIDUE) && (i== 2))
      is_selected_label = true;
    if ((mpProtView->mPickState == PICK_MEASURE) && (i== 5))
      is_selected_label = true;
    if ((mpProtView->mPickState == PICK_ANCHOR) && (i == 8))
      is_selected_label = true;
    
    if (is_selected_label)
    {
      dc.SetPen(wxColor("LIGHT GREY"));
      dc.SetBrush(wxBrush("LIGHT GREY"));
      dc.DrawRectangle(left_x, getY(), lengths[i], getHeight());
    }

    if (is_selected_label)
      dc.SetTextForeground("BLACK");
    else    
      dc.SetTextForeground("LIGHT GREY");
    text = _T(labels[i].c_str());
    dc.DrawText(text, left_x + 2, getY());


    if (isInside(mMouseX, mMouseY) && (mMouseX >= left_x) && (mMouseX < right_x))
      if ((i != 1) && (i != 4) && (i != 7))
      {
        dc.SetPen(wxPen(wxColor("GREEN"), 2, wxSOLID));
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        dc.DrawRectangle(left_x - 2, getY() - 2, lengths[i] + 4, getHeight() + 4);
      }

    left_x = right_x;
  }
}

