//
//  **************************************************************************
//
//  pdb.cpp
//  (C) 2003 Bosco K. Ho 
//
//  Adapted from the The Biomolecule Toolkit (C) 2001, Eric Alm 
//  Keeps the PDB interface separate from the rest of the code, allowing 
//  easier code maintainence 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//
 

#include <fstream>
#include <algorithm>
#include "pdb.h"


using namespace std;


class HydrogenNameBlk 
{
public:
  HydrogenNameBlk(int rResKey, 
                  std::string rPdbName, 
                  std::string rAmberName):
                  resKey(rResKey), 
                  pdbName(rPdbName), 
                  amberName(rAmberName) {};
  int resKey;
  std::string pdbName, amberName;
};

static const HydrogenNameBlk hydrogenName[] = 
{
  HydrogenNameBlk(ARG, "1HB ",	"2HB "),
  HydrogenNameBlk(ARG, "2HB ",	"3HB "),
  HydrogenNameBlk(ARG, "1HG ",	"2HG "),
  HydrogenNameBlk(ARG, "2HG ",	"3HG "),
  HydrogenNameBlk(ARG, "1HD ",	"2HD "),
  HydrogenNameBlk(ARG, "2HD ",	"3HD "),
  HydrogenNameBlk(ASP, "1HB ",	"2HB "),
  HydrogenNameBlk(ASP, "2HB ",	"3HB "),
  HydrogenNameBlk(ASN, "1HB ",	"2HB "),
  HydrogenNameBlk(ASN, "2HB ",	"3HB "),
  HydrogenNameBlk(CYS, "1HB ",	"2HB "),
  HydrogenNameBlk(CYS, "2HB ",	"3HB "),
  HydrogenNameBlk(GLU, "1HB ",	"2HB "),
  HydrogenNameBlk(GLU, "2HB ",	"3HB "),
  HydrogenNameBlk(GLU, "1HG ",	"2HG "),
  HydrogenNameBlk(GLU, "2HG ",	"3HG "),
  HydrogenNameBlk(GLN, "1HB ",	"2HB "),
  HydrogenNameBlk(GLN, "2HB ",	"3HB "),
  HydrogenNameBlk(GLN, "1HG ",	"2HG "),
  HydrogenNameBlk(GLN, "2HG ",	"3HG "),
  HydrogenNameBlk(GLY, "1HA ",	"2HA "),
  HydrogenNameBlk(GLY, "2HA ",	"3HA "),
  HydrogenNameBlk(HIS, "1HB ",	"2HB "),
  HydrogenNameBlk(HIS, "2HB ",	"3HB "),
  HydrogenNameBlk(ILE, "1HG1",  "2HG1"),
  HydrogenNameBlk(ILE, "2HG1",  "3HG1"),
  HydrogenNameBlk(LEU, "1HB ",	"2HB "),
  HydrogenNameBlk(LEU, "2HB ",	"3HB "),
  HydrogenNameBlk(LYS, "1HB ",	"2HB "),
  HydrogenNameBlk(LYS, "2HB ",	"3HB "),
  HydrogenNameBlk(LYS, "1HG ",	"2HG "),
  HydrogenNameBlk(LYS, "2HG ",	"3HG "),
  HydrogenNameBlk(LYS, "1HD ",	"2HD "),
  HydrogenNameBlk(LYS, "2HD ",	"3HD "),
  HydrogenNameBlk(LYS, "1HE ",	"2HE "),
  HydrogenNameBlk(LYS, "2HE ",	"3HE "),
  HydrogenNameBlk(MET, "1HB ",	"2HB "),
  HydrogenNameBlk(MET, "2HB ",	"3HB "),
  HydrogenNameBlk(MET, "1HG ",	"2HG "),
  HydrogenNameBlk(MET, "2HG ",	"3HG "),
  HydrogenNameBlk(PHE, "1HB ",	"2HB "),
  HydrogenNameBlk(PHE, "2HB ",	"3HB "),
  HydrogenNameBlk(PRO, " H1 ",	" H2 "),
  HydrogenNameBlk(PRO, " H2 ",	" H3 "),
  HydrogenNameBlk(PRO, "1HB ",	"2HB "),
  HydrogenNameBlk(PRO, "2HB ",	"3HB "),
  HydrogenNameBlk(PRO, "1HG ",	"2HG "),
  HydrogenNameBlk(PRO, "2HG ",	"3HG "),
  HydrogenNameBlk(PRO, "1HD ",	"2HD "),
  HydrogenNameBlk(PRO, "2HD ",	"3HD "),
  HydrogenNameBlk(SER, "1HB ",	"2HB "),
  HydrogenNameBlk(SER, "2HB ",	"3HB "),
  HydrogenNameBlk(TRP, "1HB ",	"2HB "),
  HydrogenNameBlk(TRP, "2HB ",	"3HB "),
  HydrogenNameBlk(XXX, "    ",	"    ")
};


PdbReader::~PdbReader() 
{
  if ((mFilename != "") && (mpStream != NULL))  // input was via file
    delete mpStream;
}



// Open mpStream for input, check for errors
void PdbReader::loadStream()
{
  // open the stream or file and make sure it is good
  if (mIsStreamLoaded) return;

  // input via file
  if (mFilename != "") 
  { 
    mpStream = new ifstream(mFilename.c_str());
    if (mpStream->fail()) 
      return;
  }
  else 
  { // input via passed stream
    if (mpStream->fail()) 
      return;
  }
  mIsStreamLoaded = true;
}



void PdbReader::renameAmberHydrogens()
{
  int nHydrogenType = 0;
  while (hydrogenName[nHydrogenType].resKey != XXX)
    nHydrogenType++;
    
  // scan through buffer to find one hydrogen
  // that fits only the amber naming pattern
  bool isAmberNamingConvention = false;
  for (BufferList::iterator lineIter = mBuffer.begin(); 
       lineIter < mBuffer.end(); 
       lineIter++) 
  {
    if (lineIter->substr(0,4) != "ATOM") 
      continue;  

    string name = lineIter->substr(12,4); 
    if (name[1] != 'H')
      continue;

    string resName = lineIter->substr(17,3);  
    int resKey = get_res_key(resName);

    bool isPdb = false;
    bool isAmber = false;
    for (int i=0; i<nHydrogenType; i++)
    {
      if (resKey == hydrogenName[i].resKey)
      {
        if (name == hydrogenName[i].pdbName) 
          isPdb = true;
        else if (name == hydrogenName[i].amberName)
          isAmber = true;
      }
    }
    if ((!isPdb) && isAmber)
    {  
      isAmberNamingConvention = true;
      break;
    }
  }
  
  if (!isAmberNamingConvention)
      return;  

  // renames all hydrogens to amber naming convention
  for (BufferList::iterator lineIter = mBuffer.begin(); 
       lineIter < mBuffer.end(); 
       lineIter++) 
  {
    if (lineIter->substr(0,4) != "ATOM") 
      continue;  
    string name = lineIter->substr(12,4); 
    if (name[1] != 'H')
      continue;
    string resName = lineIter->substr(17,3);  
    int resKey = get_res_key(resName);
    for (int i=0; i<nHydrogenType; i++)
    {  
      if ( (resKey == hydrogenName[i].resKey)
        && (name == hydrogenName[i].amberName) )
      {
        lineIter->replace(12, 4, hydrogenName[i].pdbName);
      }
    }
  } 
}


// load one model from the stream into mBuffer
void PdbReader::loadModel()
{
  if (mIsModelLoaded) return;

  loadStream();
  string current_line;
  getline(*mpStream, current_line);

  while (!mpStream->eof() && current_line.substr(0,6) != "ENDMDL") 
  {
    mBuffer.push_back(current_line);
    getline(*mpStream, current_line);
  }

  mIsModelLoaded = true;
}



// return the Pdb header
string PdbReader::getHeader()
{
  loadModel();

  string header;
  BufferList::iterator lineIter;
  for (lineIter = mBuffer.begin(); lineIter < mBuffer.end(); lineIter++) 
  {
    if (lineIter->substr(0,6) == "COMPND") 
    {
      int n = 60;
      while ((*lineIter)[10 + n - 1] == ' ')
        n--;  
      header += lineIter->substr(10, n);
    }
  }
  return header;
}



// return the Pdb header
string PdbReader::getPdbCode() 
{
  loadModel();

  BufferList::iterator lineIter;
  for (lineIter = mBuffer.begin(); lineIter < mBuffer.end(); lineIter++) 
  {
    if (lineIter->substr(0,6) == "HEADER") 
      return lineIter->substr(62,4);
  }
  return "";
}



// determine the chains in the current model
vector<char> PdbReader::getChainLabels()
{
  loadModel();

  vector<char> chainsIdList;
  BufferList::iterator lineIter;
  for (lineIter = mBuffer.begin(); lineIter < mBuffer.end(); lineIter++) 
  {
    if (lineIter->substr(0,4) != "ATOM") continue;  // only search ATOM records
    char ch = (*lineIter)[21]; // get the chain
    if (find(chainsIdList.begin(), chainsIdList.end(), ch) == chainsIdList.end()) 
      chainsIdList.push_back(ch); 
  }
  return chainsIdList;
}



void PdbReader::loadChain(Protein& p, const char chainLabel) 
{
  char chainId = chainLabel;
  if (chainId == '?') 
  {
    std::vector<char> pdb_chains = getChainLabels();
    chainId = pdb_chains[0];
  }

  loadModel();
  renameAmberHydrogens();

  p.clear();

  // parse the file
  BufferList::iterator lineIter;
  BufferList::iterator end = mBuffer.end();
  for (lineIter = mBuffer.begin(); lineIter < end; lineIter++) 
  {
    Atom atom;

    if (lineIter->substr(0,6) != "ATOM  ") continue;


    atom.chainId     = (*lineIter)[21]; 
    if (atom.chainId != chainId) continue;
    
    //load lineIter
    atom.no      = atoi( lineIter->substr(6,5).c_str() );
    atom.name    = lineIter->substr(12,4); 
    atom.key     = get_atom_key(atom.name);  
    atom.altLocNo    = (*lineIter)[16];
    atom.resName     = lineIter->substr(17,3);  
    atom.resNo       = atoi( lineIter->substr(22,4).c_str() );
    atom.resKey      = get_res_key(atom.resName); 
    atom.insertCode  = (*lineIter)[26];
    atom.pos         = Vector3d( atof( lineIter->substr(30,8).c_str() ),
                                 atof( lineIter->substr(38,8).c_str() ),
                                 atof( lineIter->substr(46,8).c_str() ) );
    if (lineIter->size() >= 60)
      atom.occup     = atof( lineIter->substr(54,6).c_str() );
    else
      atom.occup     = DEFAULT_OCCUPANCY;

    if (lineIter->size() >= 66) 
      atom.bFactor   = atof( lineIter->substr(60,6).c_str() );
    else 
      atom.bFactor   = DEFAULT_B_FACTOR;
    
    atom.isOn        = true;     

    // Skip altLoc positions:
    if ( atom.altLocNo != ' ' && atom.altLocNo != 'A' &&
         atom.altLocNo != '1')
      continue;
      
    p.checkAtomName(atom);

    p.pushAtom(atom);
  }

  p.scanRes();
  p.extractAngles();
  p.setName(getHeader());
  p.setPdbCode(getPdbCode());

}


PdbWriter::~PdbWriter() 
{
  if ((mFilename != "") && (mpStream != NULL))  // input was via file
    delete mpStream;
}



// Open mpStream for input, check for errors
void PdbWriter::loadStream()
{
  // open the stream or file and make sure it is good
  if (mIsStreamLoaded) return;

  // input via file
  if (mFilename != "") 
  { 
    mpStream = new ofstream(mFilename.c_str());
    if (mpStream->fail()) 
      return;
  }
  else 
  { // input via passed stream
    if (mpStream->fail()) 
      return;
  }
  mIsStreamLoaded = true;
}



void PdbWriter::writeChain(Protein& protein, bool isHydrogen) 
{
  if (!mIsStreamLoaded) return;
  int i = 1;
  for (AtomIter iter = protein.firstAtomIter(); iter != protein.lastAtomIter(); iter++) 
  {
    Atom& a = *iter;
    if ( !(!isHydrogen && (a.element == 'H')) )
      *mpStream << "ATOM " 
                << setw(6) << i << " "
                << setw(4) << a.name << " " 
                << setw(3) << a.resName << " "
                << setw(5) << a.resNo << "    "
                << std::fixed << setw(8) << setprecision(3) << a.pos.x() 
                << std::fixed << setw(8) << setprecision(3) << a.pos.y() 
                << std::fixed << setw(8) << setprecision(3) << a.pos.z() 
                << std::fixed << setw(6) << setprecision(2) << 1.00 
                << std::fixed << setw(6) << setprecision(2) << 0.0 << endl;
    i++;
  }
}
