//
//  **************************************************************************
//
//  protview.cpp
//  (C) 2003 Bosco K. Ho 
//
//  Basic definition of a screen/mouse widget. Simple widgets are
//  included: clickable, toggle and state buttons. As well, a mouse drawing
//  function is included that displays text.  
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//


#include "widget.h"
#include "dialog.h"
#include "event.h"
#include "protview.h"

#include <vector>
#include <algorithm>

#define DSTYLE_HBOND 1
#define DSTYLE_BOND  2
#define DSTYLE_DISULFIDE 3
#define DSTYLE_STERIC_CLASH 4
#define DSTYLE_WEAK_HBOND 5



///////////////////////////////////////////////////
// ProtView: widget to display protein
///////////////////////////////////////////////////

ProtView::ProtView(wxWindow* pParent, 
                   MyProtein* pProtein, 
                   Controller* pController): 
  Widget(pParent, 0, 0, -1, -1), 
  mpController(pController),
  mpProtein(pProtein),

  mIsBallStick(true),
  mIsCaTrace(false),
  mIsCaSidechain(false),
  mIsEverything(true),
  mIsBackbone(false),

  mIsHbonds(true),
  mIsWeakHbonds(true),
  mIsStericClash(true),

  mIsSlab(false),
  mSlabThickness(mpProtein->maxLength()),
  mIsZDepth(true),

  mIsHydrogen(false),
  mIsPeptide(true),
  
  mPickState(PICK_RESIDUE) 
  
{
  initializeFocus();
  mDisplayAtomRadius = 0.25;
  mDisplayLineWidth  = 0.05;
}



///////////////////////////////
// Mouse handling code
///////////////////////////////

void ProtView::onLeftMouseDClick()
{
  if (mHighlight.isValid())
  {
    mpController->setFocus(mHighlight);
    Refresh();
  }
}



void ProtView::onLeftMouseDown()
{
  if (mHighlight.isValid())
    mPress = mHighlight;
  else
    mPress.res = -1;
  mSaveMouseX = mMouseX;
  mSaveMouseY = mMouseY;
}






void ProtView::onLeftMouseUp()
{
  if ((mHighlight.isValid()) && (mHighlight == mPress))
    {
      if (mPickState == PICK_FOCUS)
        mpController->setFocus(mHighlight);
      else if (mPickState == PICK_RESIDUE)
        mpController->select(mPress.res);
      else if (mPickState == PICK_MEASURE)
        mpController->toggleMeasure(mPress);
      else if (mPickState == PICK_ANCHOR)
        mpController->toggleLoopClosureRes(mPress.res);
      Refresh();
    }
  mSaveMouseX = mMouseX;
  mSaveMouseY = mMouseY;
}



void ProtView::onLeftMouseDrag()
{
  if (mSaveMouseX > 0)
  {
    double rotate = (mMouseX - mSaveMouseX) / 3.0;
    mCameraLook.rotate(-rotate, mCameraUp);
   
    Vector3d norm = cross(mCameraLook, mCameraUp);
   
    double pitch  = (mMouseY - mSaveMouseY) / 3.0;
    mCameraLook.rotate(-pitch, norm);
    mCameraUp.rotate(-pitch, norm);
   
    mSaveMouseY = mMouseY;
    mSaveMouseX = mMouseX;

    Refresh();
  }
}



void ProtView::onRightMouseDown()
{
  if (mHighlight.isValid())
    mPress = mHighlight;
  else
  {
    mSaveMouseX = mMouseX;
    mSaveMouseY = mMouseY;
  }
}


void ProtView::onRightMouseUp()
{
  if ((mHighlight.isValid()) && (mHighlight == mPress))
  {
    mSaveMouseX = -1;
    mSaveMouseY = -1;
    mpController->setInsertRes(mHighlight.res);
    mpController->setSelect(mHighlight.res, true);
    Refresh();
    PopupMenu popupMenu(mpParent, mpController);
    popupMenu.show(mMouseX, mMouseY);
    Refresh();
  }
}


void ProtView::onRightMouseDrag()
{
  if (mSaveMouseX > 0)
  {
     double new_zoom = (mCameraLook.length() + 0.25*(mMouseY - mSaveMouseY));
     if (new_zoom > 1.5) 
       mCameraLook.scaleTo(new_zoom);
 
     mCameraUp.rotate(0.5*(mMouseX - mSaveMouseX), mCameraLook);
 
     computeCameraFromFocus();
   
     mSaveMouseY = mMouseY;
     mSaveMouseX = mMouseX;

     Refresh();
  }
}



///////////////////////////////////////////////////////
// Camera manipulation code
///////////////////////////////////////////////////////

void ProtView::initializeFocus(void)
{
  if (mpProtein->isEmpty()) 
    return;

  mpProtein->computeCenterOfMass();
  Vector3d centerOfMass = mpProtein->getCenterOfMass();

  int center_res = 0;
  double least = 0.0;
  for (int i = 0; i < mpProtein->nRes(); i++)
  {
    double new_least = pos_dist_squared(centerOfMass, mpProtein->atom(i, CA).pos);
    if ((i==0) || (new_least < least))
    {
      center_res = i;
      least = new_least;
    }
  }
  mpController->setFocus(AtomMarker(mpProtein, center_res, CA));

  resetFocus();

  mCameraLook = mFocusAxis.scaled(-mpProtein->maxLength());
  mCameraUp = mFocusUp.scaled(1.0);

  computeCameraFromFocus();
 
}



// computes a new transformation matrix based on the given
// focus position and camera-look and camera-up vectors

void ProtView::computeCameraFromFocus(void)
{
  mCameraPos = mFocusPos - mCameraLook;
  mProject3d.setCamera(mCameraPos, mCameraLook, mCameraUp);
}



// given the mFocus.res and mFocus.atom values, changes the focusPos
// to that mFocus.atom. but if the option is to center then calculates
// the current center of mass based on user selected options
// once the focus is calculated, does house-keeping, calculates the
// new camera position and counts H-bonds

void ProtView::resetFocus(void)
{
  if (mpProtein->isEmpty()) return;

  mFocusPos = mpProtein->mFocus.pos();

  int focus_res = mpProtein->mFocus.res;
  Vector3d ca_n = mpProtein->atom(focus_res, N).pos - mpProtein->atom(focus_res, CA).pos;
  Vector3d ca_c = mpProtein->atom(focus_res, C).pos - mpProtein->atom(focus_res, CA).pos;

  mFocusUp      = -(ca_n + ca_c);
  mFocusAxis    = cross(ca_n, ca_c);

  computeCameraFromFocus();
}



// since we have a way of defining a standard reference frame w.r.t to each amino acid
// backbone, this function jumps to another focus_position but tries to
// maintain the relative orientation of the camera to the standard reference frame

void ProtView::resetFocusWithSameOrientation(void)
{
  // get relative angles of old focus 
  double camera_look_a = vec_angle(mFocusUp, mCameraLook);
  double camera_look_b = vec_dihedral(mCameraLook, mFocusUp, mFocusAxis);

  double zoom   = mCameraLook.length();
  double up_a   = vec_dihedral(mCameraUp, mCameraLook, mFocusUp);

  resetFocus();

  // reconstruct mCameraLook and mCameraUp using the relative angles of old focus 
  mCameraLook = mFocusAxis.rotated(camera_look_b, mFocusUp);

  Vector3d perp = cross(mFocusUp, mCameraLook);
  mCameraLook.rotate(-(90 - camera_look_a), perp);
  mCameraLook.scaleTo(zoom);

  perp = cross(mFocusUp, mCameraLook);
  mCameraUp = mCameraLook.rotated(-90, perp);
  mCameraUp.rotate(up_a, mCameraLook);
  computeCameraFromFocus();
}



///////////////////////////////////////////////////////
// Drawing code
///////////////////////////////////////////////////////

void ProtView::draw(wxBufferedPaintDC &dc)
{
  if (!mpProtein->mFocus.isValid())
    initializeFocus();

  // render 3d objects to 2d
  wxSize sz = GetClientSize();
  mProject3d.setViewport(0, 0, sz.GetWidth(), sz.GetHeight());
  renderProtein();
  renderMeasure();
  renderAnchor();
  renderHighlight(mpProtein->mFocus.pos(), PURPLE);
  if (mHighlight.isValid())
    renderHighlight(mHighlight.pos(), GREEN);

  // now draw 2d objects to the wxWindows drawing-context dc
  dc.SetBackground(*wxBLACK);
  dc.Clear();
  if (mIsZDepth)
    mProject3d.drawZFixLight(dc, mFocusPos);
  else
    mProject3d.drawZFlat(dc);
  if (mHighlight.isValid())
    renderHighlightInfoBox(dc);

  wxString text;
  text = _T("Right mouse: zoom/roll/edit");
  dc.SetFont( wxFont(10, wxMODERN, wxNORMAL, wxNORMAL) );
  long length, height, descent;
  dc.GetTextExtent( text, &length, &height, &descent );
  dc.SetBackgroundMode(wxTRANSPARENT);
  dc.SetTextForeground( wxColor("LIGHT GREY") );
  dc.DrawText(text, 5, getY() + getHeight() - height - 6);
}



void ProtView::renderHighlightInfoBox(wxBufferedPaintDC &dc)
{   
  long length, height, descent;

  wxString text;
  text = _T(mpProtein->resAtomName(mHighlight.res, mHighlight.atom).c_str());

  dc.SetFont( wxFont(10, wxMODERN, wxNORMAL, wxNORMAL) );
  dc.GetTextExtent( text, &length, &height, &descent );

  dc.SetPen( wxPen( wxT("black"), 1, wxSOLID) );
  dc.SetBrush( *wxWHITE_BRUSH );
  dc.DrawRectangle( mMouseX, mMouseY + 22, length + 6, height + 6);

  dc.SetBackgroundMode(wxTRANSPARENT);
  dc.SetTextForeground( *wxBLACK );
  dc.DrawText( text, mMouseX + 3, mMouseY + 25);    
}



bool ProtView::isPosInSlab(Vector3d& test_pos)
{
  Vector3d& focus_pos = mpProtein->mFocus.pos();
  
  double focus_z = mProject3d.point(focus_pos).z();
  double test_z = mProject3d.point(test_pos).z();
  double diff = test_z - focus_z;
  
  return ((diff > -mSlabThickness) && (diff < mSlabThickness));
}



int ProtView::atomColor(char element)
{
  switch (element)
  {
    case 'C':
    case 'H':
      return LIGHT_GREY;
      break;
    case 'O':
      return  RED;
      break;
    case 'N':
      return  BLUE;
      break;
    case 'S':
      return  YELLOW;
      break;
  }
  return LIGHT_GREY;
}



//////////////////////////////////////////////////
// Rendering code
//////////////////////////////////////////////////


void ProtView::renderMeasure()
{
  int n = mpProtein->mMeasure.size();
  if (0 == n) return;

  int color = YELLOW;
  Vector3d pos[4];

  for (int i = 0; i < n; i++)
    pos[i] = mpProtein->mMeasure.pos(i);

  for (int i = 0; i < n-1; i++)
    mProject3d.dashLine(pos[i], pos[i+1], color, 0.1);

  for (int i = 0; i < n; i++) 
    mProject3d.ring(pos[i], mDisplayAtomRadius * 1.5, 
                    mDisplayAtomRadius * 0.1, color);
  
  mpController->makeMeasurements();
  
  if (n >= 2)
  {
    Vector3d p = (pos[0] + pos[1]) * 0.5;
    mProject3d.string(p, mpProtein->mMeasure.mLabelList[0], color);
  }
  
  if (n >= 3)
    mProject3d.string(pos[1], mpProtein->mMeasure.mLabelList[1], color);
    
  if (n == 3)
  {
    Vector3d p = (pos[1] + pos[2]) * 0.5;
    mProject3d.string(p, mpProtein->mMeasure.mLabelList[2], color);
  }
  else if (n >= 4)
  {
    Vector3d p = (pos[1] + pos[2]) * 0.5;
    mProject3d.string(p, mpProtein->mMeasure.mLabelList[2], color);
    mProject3d.string(pos[2], mpProtein->mMeasure.mLabelList[3], color);
    p = (pos[2] + pos[3]) * 0.5;
    mProject3d.string(p, mpProtein->mMeasure.mLabelList[4], color);
  }
}



void ProtView::renderAnchor()
{
  int n = mpProtein->mLoopClosure.nSelectedRes();
  int color = PURPLE;

  Vector3d pos[3];
  for (int j=0; j < n; j++)
  {
    int k = mpProtein->mLoopClosure.res(j);
    pos[j] = mpProtein->res(k).atom(CA).pos;
  }
  
  if (n == 2)
    mProject3d.solidLine(pos[0], pos[1], color, 2.0*mDisplayLineWidth);

  if (n == 3)
  {
    mProject3d.solidLine(pos[0], pos[1], color, 2.0*mDisplayLineWidth);
    mProject3d.solidLine(pos[1], pos[2], color, 2.0*mDisplayLineWidth);
    mProject3d.solidLine(pos[0], pos[2], color, 2.0*mDisplayLineWidth);
  }

  for (int i = 0; i < n; i++) 
    mProject3d.solidSphere(pos[i], mDisplayAtomRadius * 2.0, color);
}



void ProtView::renderProtein()
{
  int nRes = mpProtein->nRes();

  if (mpProtein->mIsChanged) 
  {
    resetFocusWithSameOrientation();
    mpProtein->mIsChanged = false;
  }
  else
    resetFocus();
  
  mProject3d.clear();

  for (int i = 1; i < nRes; i++)
  {
    if ( mIsSlab && !isPosInSlab(mpProtein->res(i-1).atom(CA).pos)
                 && !isPosInSlab(mpProtein->res(i).atom(CA).pos) )
      continue;
    renderResidueConnector(i-1, i);
  }
  
  if (mIsEverything || mIsBackbone)
    renderBonds(&(mpProtein->mPeptideBonds), DSTYLE_BOND);
  
  if (mIsCaSidechain || mIsEverything)
    renderBonds(&(mpProtein->mDisulfideBonds), DSTYLE_DISULFIDE);
  else if (mIsBackbone)
  {
    mIsBackbone = false;
    mIsEverything = true;
    renderBonds(&(mpProtein->mDisulfideBonds), DSTYLE_DISULFIDE);
    mIsBackbone = true;
    mIsEverything = false;
  }

  // draw interactions
  if (!mIsCaTrace)
  {
    if (mIsStericClash)
      renderBonds(&(mpProtein->mStericClashes), DSTYLE_STERIC_CLASH);

    if (mIsHydrogen)
    {
       if (mIsHbonds)
         renderBonds(&mpProtein->mWithHydrogenHbonds, DSTYLE_HBOND);
       if (mIsWeakHbonds)   
         renderBonds(&mpProtein->mWithHydrogenWeakHbonds, DSTYLE_WEAK_HBOND);
    }
    else
    {
      if (mIsHbonds)
        renderBonds(&mpProtein->mHeavyAtomHbonds, DSTYLE_HBOND);
      if (mIsWeakHbonds)   
        renderBonds(&mpProtein->mHeavyAtomWeakHbonds, DSTYLE_WEAK_HBOND);
    }
  }

  // renderAtomMarker() will check mouse to update mHighlight.res
  mHighlight.turnOff();

  // draw residues
  for (int i = 0; i < nRes; i++)
  {
    if (!mIsCaTrace)
    {  
      if (mIsBackbone && (mpProtein->resKey(i) == PRO))
      {
        mIsBackbone = false;
        mIsEverything = true;
        renderBonds(&(mpProtein->mResBonds[i]), DSTYLE_BOND);
        mIsBackbone = true;
        mIsEverything = false;
      }
      else if (mIsBackbone && (mpProtein->resKey(i) == CYS))
      {
        mIsBackbone = false;
        mIsEverything = true;
        renderBonds(&(mpProtein->mResBonds[i]), DSTYLE_BOND);
        mIsBackbone = true;
        mIsEverything = false;
      }
      else
        renderBonds(&(mpProtein->mResBonds[i]), DSTYLE_BOND);
    }
  
    // draw the spheres for atoms
    renderAtomsInRes(i);
  }
}



void ProtView::renderBond(Bond* p_bond)
{
  int i = p_bond->mAtom1.res;
  int j = p_bond->mAtom2.res;

  Vector3d& p1 = p_bond->mAtom1.pos();
  Vector3d& p2 = p_bond->mAtom2.pos();
  
  if (mIsBallStick)
  {
    // display the neutral bond color in green
    int color;
    if (mpProtein->mLoopClosure.isSelectedRes(i) || mpProtein->mLoopClosure.isSelectedRes(j))
      color = PURPLE;
    else if (mpProtein->mSelect.isSelected(i) || mpProtein->mSelect.isSelected(j))
      color = GREEN;
    else
      color = GREY;
    mProject3d.solidLine(p1, p2, color, mDisplayLineWidth);
  }
  else
  {
    Vector3d mid = (p1 + p2) * 0.5;

    int color1 = atomColor(p_bond->mAtom1.element());
    int color2 = atomColor(p_bond->mAtom2.element());

    // changed the color of selected C atoms to green
    if (mpProtein->mSelect.isSelected(i) || mpProtein->mSelect.isSelected(j))
    {
      if (color1 == LIGHT_GREY)
        color1 = GREEN;
      if (color2 == LIGHT_GREY)
        color2 = GREEN;
    }
    mProject3d.line(p1, mid, color1, 0.1);
    mProject3d.line(p2, mid, color2, 0.1);
  }
}



void ProtView::renderFatBond(Bond* p_bond)
{
  int i = p_bond->mAtom1.res;
  int j = p_bond->mAtom2.res;

  Vector3d& p1 = p_bond->mAtom1.pos();
  Vector3d& p2 = p_bond->mAtom2.pos();
  
  if (mIsBallStick)
  {
    // display the neutral bond color in green
    if (mpProtein->mSelect.isSelected(i) || mpProtein->mSelect.isSelected(j))
      mProject3d.solidLine(p1, p2, GREEN, mDisplayLineWidth*3);
    else
      mProject3d.solidLine(p1, p2, GREY, mDisplayLineWidth*3);
  }
  else
  {
    Vector3d mid = (p1 + p2) * 0.5;

    int color1 = atomColor(p_bond->mAtom1.element());
    int color2 = atomColor(p_bond->mAtom2.element());

    // changed the color of selected C atoms to green
    if (mpProtein->mSelect.isSelected(i) || mpProtein->mSelect.isSelected(j))
    {
      if (color1 == LIGHT_GREY)
        color1 = GREEN;
      if (color2 == LIGHT_GREY)
        color2 = GREEN;
    }
    mProject3d.line(p1, mid, color1, 0.3);
    mProject3d.line(p2, mid, color2, 0.3);
  }
}



void ProtView::renderDisulfideBond(Bond* p_bond)
{
  Vector3d& p1 = p_bond->mAtom1.pos();
  Vector3d& p2 = p_bond->mAtom2.pos();
  
  if (mIsBallStick)
    mProject3d.solidLine(p1, p2, YELLOW, mDisplayLineWidth);
  else
    mProject3d.line(p1, p2, YELLOW, 0.1);
}



void ProtView::renderHbond(Bond* p_bond, int color)
{
  mProject3d.dashLine(p_bond->mAtom1.pos(),
                      p_bond->mAtom2.pos(),
                      color, 0.1);
}



void ProtView::renderBonds(BondList* pBonds, int style)
{
  int size = pBonds->size();
  for (int k=0; k<size; k++)
  {
    Bond* bond = (*pBonds)[k];
    
    if (mIsSlab && !isPosInSlab(bond->mAtom1.pos()) 
                && !isPosInSlab(bond->mAtom2.pos()))
      continue;

    if (mIsCaSidechain)
      if ((bond->mAtom1.atom < CB) && (bond->mAtom2.atom < CB))
        continue;
          
    if (mIsBackbone)
      if ((bond->mAtom1.atom > CB) || (bond->mAtom2.atom > CB))
        continue;

    if (!mIsHydrogen)
      if (bond->mAtom1.element() == 'H' || 
          bond->mAtom2.element() == 'H')
        continue;    

    switch (style)
    {
      case DSTYLE_HBOND:
        renderHbond(bond, ORANGE);
        break;
      case DSTYLE_WEAK_HBOND:
        renderHbond(bond, LIGHT_ORANGE);
        break;
      case DSTYLE_STERIC_CLASH:
        renderHbond(bond, WHITE);
        break;
      case DSTYLE_BOND:
        renderBond(bond);
        break;
      case DSTYLE_DISULFIDE:
        renderDisulfideBond(bond);
    }
  }
}




void ProtView::renderHighlight(Vector3d& pos, int color)
{
  mProject3d.ring(pos, mDisplayAtomRadius * 2.5, mDisplayAtomRadius * 0.6, color);
}



void ProtView::renderPeptidePlane(int i, int j)
{
  Vector3d v[4];

  if (mpProtein->hasAtom(i, CA) &&
      mpProtein->hasAtom(i, O) &&
      mpProtein->hasAtom(j, CA))
  {
      v[0] = mpProtein->atom(i, CA).pos;
      v[1] = mpProtein->atom(i, O).pos;
      v[2] = mpProtein->atom(j, CA).pos;

      if (mpProtein->resKey(j) == PRO)
      {
        if (mpProtein->hasAtom(j, CD1))
          v[3] = mpProtein->atom(j, CD1).pos;
      }
      else
      {
        if (mpProtein->hasAtom(j, H))
          v[3] = mpProtein->atom(j, H).pos;
        else
          v[3] = mpProtein->atom(j, N).pos;  
      }
  }

  mProject3d.solidQuad(v, DARK_GREY);
}



void ProtView::renderResidueConnector(int i, int j)
{
  if (mIsCaTrace || mIsCaSidechain)
  {  
    if (mIsCaSidechain && (mpProtein->resKey(j) == PRO))
    {
      Bond b(AtomMarker(mpProtein, i, CA), AtomMarker(mpProtein, j, N));
      renderFatBond(&b);
      Bond c(AtomMarker(mpProtein, j, CA), AtomMarker(mpProtein, j, N));
      renderBond(&c);
    }
    else
    {
      Bond b(AtomMarker(mpProtein, i, CA), AtomMarker(mpProtein, j, CA));
      renderFatBond(&b);
    }
  }
  else
  {
    if (mIsPeptide && !mIsCaTrace && !mIsCaSidechain)
      if ((j-i) == 1)
        renderPeptidePlane(i, j);
  }
}



void ProtView::renderAtomMarker(AtomMarker atom_marker)
{
  if (mIsSlab && !isPosInSlab(atom_marker.pos()))
    return;

  if (!mIsHydrogen)
    if (atom_marker.element() == 'H')
      return;
  
  if (mIsBallStick)
    mProject3d.solidSphere(atom_marker.pos(), mDisplayAtomRadius, 
                           atomColor(atom_marker.element()));

  int zOut;
  if (mProject3d.isXYOverSphere(atom_marker.pos(), mDisplayAtomRadius*2.5, 
                                mMouseX, mMouseY, &zOut))
  {
    if ( (!mHighlight.isValid()) || (zOut < mHighlightZ) )
    {
      mHighlight.set(atom_marker);
      mHighlightZ    = zOut;
    }
  }
}



void ProtView::renderAtomsInRes(int i)
{
  if (mIsCaTrace)
  {
    if (mpProtein->hasAtom(i, CA))
      renderAtomMarker(AtomMarker(mpProtein, i, CA));
    return;
  }
  
  
  if (mIsCaSidechain)
  {
    int n_atom = mpProtein->nAtomInRes(i);
    for (int j=0; j < n_atom; j++)
    {  
       int atom_key = mpProtein->res(i).atomIters[j]->key;
       if (atom_key >= CB)
         renderAtomMarker(AtomMarker(mpProtein, i, atom_key));
       renderAtomMarker(AtomMarker(mpProtein, i, CA));
       if (mpProtein->resKey(i) == PRO)
         renderAtomMarker(AtomMarker(mpProtein, i, N));  
    }
    return;
  }

  if (mIsBackbone)
  {
    if ((mpProtein->resKey(i) != PRO) && (mpProtein->resKey(i) != CYS))
    {
      int n_atom = mpProtein->nAtomInRes(i);
      for (int j=0; j < n_atom; j++)
      {  
       int atom_key = mpProtein->mResList[i].atomIters[j]->key;
         if (atom_key <= CB)
           renderAtomMarker(AtomMarker(mpProtein, i, atom_key));
      }
      return;
    }
  }   
  
  // mIsEverything 
  int n_atom = mpProtein->nAtomInRes(i);
  for (int j=0; j < n_atom; j++)
  {  
    int atom_key = mpProtein->mResList[i].atomIters[j]->key;
    renderAtomMarker(AtomMarker(mpProtein, i, atom_key));
  }
}



