/////////////////////////////////////////////////////////////////////////////
// Name:        Ramachandran Plot Explorer
// Purpose:     Manipulate proteins
// Author:      Bosco Ho
// Modified by:
// Copyright:   (c) 2005 Bosco Ho
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#if defined(__GNUG__) && !defined(__APPLE__)
    #pragma implementation
    #pragma interface
#endif

#include "wx/wx.h"
#include "wx/app.h"
#include "wx/frame.h"
#include "wx/menu.h"
#include "wx/msgdlg.h"
#include "wx/dcbuffer.h"
#include "wx/image.h"
#include "wx/artprov.h"
#include "wx/filedlg.h"

#include "myprot.h"
#include "controller.h"
#include "widget.h"
#include "dialog.h"
#include "protview.h"
#include "controls.h"

// stores the id-keys to menu commands so all modules can access them
#include "event.h"

/////////////////////////////////////////////////////////////////////
// View - the wxWidget that handles the display of controls and model
/////////////////////////////////////////////////////////////////////

class View: public wxWindow
{
public:
  View(wxFrame *pParent):
     wxWindow(pParent, wxID_ANY, wxDefaultPosition, 
              wxDefaultSize, wxFULL_REPAINT_ON_RESIZE),
     mpParent(pParent), 
     miLeftMouse(-1), 
     miRightMouse(-1) {}
  ~View() { clearWidgets(); }

  void clearWidgets();
  void addWidget(Widget* w) { mWidgetList.push_back(w); }

  void draw(wxBufferedPaintDC& dc);

  void OnPaint(wxPaintEvent& event);
  void OnLeaveWindow(wxMouseEvent& WXUNUSED(event)) { Refresh(); }
  void OnEraseBackground(wxEraseEvent& WXUNUSED(event)) {}
  void OnMouseMove(wxMouseEvent &event);
  void onRightMouseDown(wxMouseEvent& event);
  void onRightMouseUp(wxMouseEvent& event); 
  void onLeftMouseDown(wxMouseEvent& event);
  void onLeftMouseDClick(wxMouseEvent& event);
  void onLeftMouseUp(wxMouseEvent& event); 

  wxFrame* mpParent;

  std::vector<Widget*> mWidgetList;
  
  int miLeftMouse;
  int miRightMouse;
  
  int mMouseX;
  int mMouseY;

  DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(View, wxWindow)
    EVT_PAINT(View::OnPaint)
    EVT_MOTION(View::OnMouseMove)
    EVT_LEFT_DOWN(View::onLeftMouseDown)
    EVT_LEFT_UP(View::onLeftMouseUp)
    EVT_LEFT_DCLICK(View::onLeftMouseDClick)
    EVT_RIGHT_DOWN(View::onRightMouseDown)
    EVT_RIGHT_UP(View::onRightMouseUp)
    EVT_ERASE_BACKGROUND(View::OnEraseBackground)
    EVT_LEAVE_WINDOW(View::OnLeaveWindow)
END_EVENT_TABLE()



void View::OnPaint(wxPaintEvent& WXUNUSED(event))
{
  wxBufferedPaintDC dc(this);
  for (unsigned int i = 0; i < mWidgetList.size(); i++) 
    if (mWidgetList[i]->isOn())
      mWidgetList[i]->draw(dc);
}



void View::clearWidgets()
{
  for (unsigned int i = 0; i < mWidgetList.size(); i++) 
    delete mWidgetList[i];
  mWidgetList.clear();
}



void View::OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);

  wxPoint pos = event.GetPosition();
  mMouseX = pos.x;
  mMouseY = pos.y;

  int nWidget = mWidgetList.size();
  for (int i = nWidget-1; i >= 0; i--) 
  {
    mWidgetList[i]->OnMouseMove(mMouseX, mMouseY);
    if ((miLeftMouse == i) && (event.LeftIsDown()))
      mWidgetList[i]->onLeftMouseDrag();
    if ((miRightMouse == i)&& (event.RightIsDown()))
      mWidgetList[i]->onRightMouseDrag();
  }

  Refresh();
  event.Skip();
}



void View::onLeftMouseDown(wxMouseEvent& WXUNUSED(event))
{
  int nWidget = mWidgetList.size();
  for (int i = nWidget-1; i >= 0; i--) 
    if (mWidgetList[i]->isOn())
      if (mWidgetList[i]->isInside(mMouseX, mMouseY)) 
      {
          miLeftMouse = i;
          mWidgetList[i]->onLeftMouseDown();
          break;
      }
}
  


void View::onLeftMouseDClick(wxMouseEvent& WXUNUSED(event))
{
  int nWidget = mWidgetList.size();
  for (int i = nWidget-1; i >= 0; i--) 
    if (mWidgetList[i]->isInside(mMouseX, mMouseY)) 
    {
       mWidgetList[i]->onLeftMouseDClick();
       break;
    }
}



void View::onLeftMouseUp(wxMouseEvent& WXUNUSED(event)) 
{
  if (miLeftMouse > -1)
  {
    mWidgetList[miLeftMouse]->onLeftMouseUp();
    miLeftMouse = -1;
  }
}



void View::onRightMouseDown(wxMouseEvent& WXUNUSED(event))
{
  int nWidget = mWidgetList.size();
  for (int i = nWidget-1; i >= 0; i--) 
    if (mWidgetList[i]->isInside(mMouseX, mMouseY)) 
    {
       miRightMouse = i;
       mWidgetList[i]->onRightMouseDown();
       break;
    }
}
  


void View::onRightMouseUp(wxMouseEvent& WXUNUSED(event))
{
  if (miRightMouse > -1)
  {
    mWidgetList[miRightMouse]->onRightMouseUp();
    miRightMouse = -1;
  }
}



/////////////////////////////////////////////////////////////////////
// Frame - where the Model-View-Controller meet
/////////////////////////////////////////////////////////////////////

class Frame: public wxFrame
{
public:
    Frame();

    void OnShow(wxCommandEvent &event);

    MyProtein    mProtein;
    View*        mpView;
    Controller*  mpController;

    ProtView*    mpProtView;
    RamaPlot*    mpRamaPlot;
    ChiBox*      mpChiBox;
    RamaPlotSizer* mpRamaPlotSizer;
    Sequence*    mpSequence;
    FocusPicker* mpFocusPicker;
    Scrollbar*   mpScrollbar;
    SlabBar*     mpSlabBar;
    LoopSolutions* mpLoopSolutions;
    PickControl* mpPickControl;

    void loadProtein(std::string fname);
    
protected:
    void OnQuit(wxCommandEvent& event);
    void OnOpen(wxCommandEvent& event);
    void OnSave(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);

    void OnUndo(wxCommandEvent& event);

    void OnSelect(wxCommandEvent& event);

    void OnAlaninePeptide(wxCommandEvent& event);
    void OnAlphabetProtein(wxCommandEvent& event);

    void OnBackbone(wxCommandEvent& event);
    void OnMutateRes(wxCommandEvent& event);
    void OnBuildHydrogen(wxCommandEvent& event);
    void OnBlockTerminals(wxCommandEvent& event);
    void OnChargeTerminals(wxCommandEvent& event);

    void OnDisplayDrawStyle(wxCommandEvent& event);
    void OnDisplayShow(wxCommandEvent& event);
    void OnDisplayOptions(wxCommandEvent& event);


private:
    wxString mFname;
    wxString mDir;
    
    DECLARE_EVENT_TABLE()
};


// Implementation of MyFrame
// Event table goes *with* the implementation, not the interface

BEGIN_EVENT_TABLE(Frame, wxFrame)
    EVT_MENU(EVENT_FILE_QUIT, Frame::OnQuit)
    EVT_MENU(EVENT_FILE_OPEN, Frame::OnOpen)
    EVT_MENU(EVENT_FILE_SAVE, Frame::OnSave)
    EVT_MENU(EVENT_ABOUT, Frame::OnAbout)

    EVT_MENU(EVENT_UNDO, Frame::OnUndo)

    EVT_MENU(EVENT_CLEAR_SELECTION, Frame::OnSelect)
    EVT_MENU(EVENT_SELECT_ALL, Frame::OnSelect)
    EVT_MENU(EVENT_CUT, Frame::OnSelect)
    EVT_MENU(EVENT_COPY, Frame::OnSelect)
    EVT_MENU(EVENT_DELETE, Frame::OnSelect)
    EVT_MENU(EVENT_INSERT, Frame::OnSelect)
    EVT_MENU(EVENT_INSERT_ALA, Frame::OnSelect)
    EVT_MENU(EVENT_INSERT_RIGHT, Frame::OnSelect)

    EVT_MENU(EVENT_COPY_TO_PDB, Frame::OnSelect)
    EVT_MENU(EVENT_INSERT_FROM_PDB, Frame::OnSelect)

    EVT_MENU(EVENT_BACKBONE, Frame::OnBackbone)
    EVT_MENU(EVENT_BUILD_HYDROGEN, Frame::OnBuildHydrogen)
    EVT_MENU(EVENT_BLOCK_TERMINALS, Frame::OnBlockTerminals)
    EVT_MENU(EVENT_CHARGE_TERMINALS, Frame::OnChargeTerminals)
    EVT_MENU(EVENT_ALANINE_PEPTIDE, Frame::OnAlaninePeptide)
    EVT_MENU(EVENT_ALPHABET_PROTEIN, Frame::OnAlphabetProtein)

    EVT_MENU(EVENT_DISPLAY_LINE, Frame::OnDisplayDrawStyle)
    EVT_MENU(EVENT_DISPLAY_SOLID, Frame::OnDisplayDrawStyle)

    EVT_MENU(EVENT_DISPLAY_BACKBONE, Frame::OnDisplayShow)
    EVT_MENU(EVENT_DISPLAY_EVERYTHING, Frame::OnDisplayShow)
    EVT_MENU(EVENT_DISPLAY_CA_TRACE, Frame::OnDisplayShow)
    EVT_MENU(EVENT_DISPLAY_CA_SIDECHAIN, Frame::OnDisplayShow)
  
    EVT_MENU(EVENT_DISPLAY_HYDROGEN, Frame::OnDisplayOptions)
    EVT_MENU(EVENT_DISPLAY_PEPTIDE, Frame::OnDisplayOptions)
    EVT_MENU(EVENT_DISPLAY_HBOND, Frame::OnDisplayOptions)
    EVT_MENU(EVENT_DISPLAY_WEAK_HBOND, Frame::OnDisplayOptions)
    EVT_MENU(EVENT_DISPLAY_STERIC_CLASH, Frame::OnDisplayOptions)
    EVT_MENU(EVENT_DISPLAY_Z_DEPTH, Frame::OnDisplayOptions)

    EVT_MENU(EVENT_MUTATE_POPUP, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_ALA, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_CYS, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_ASP, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_GLU, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_PHE, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_GLY, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_HIS, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_ILE, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_LYS, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_LEU, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_MET, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_ASN, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_PRO, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_GLN, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_ARG, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_SER, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_THR, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_VAL, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_TRP, Frame::OnMutateRes)
    EVT_MENU(EVENT_MUTATE_TYR, Frame::OnMutateRes)
END_EVENT_TABLE()



Frame::Frame() : wxFrame((wxFrame *)NULL, wxID_ANY, _T("Ramachandran Plot Explorer 1.0"),
                          wxDefaultPosition, wxSize(620, 440))
{
    // create the file menu
    wxMenu* p_file_menu = new wxMenu;
    p_file_menu->Append(EVENT_FILE_OPEN, _T("&Open\tCtrl-o"), _T("Open a PDB file"));
    p_file_menu->Append(EVENT_FILE_SAVE, _T("&Save As\tCtrl-s"), _T("Save current protein as PDB file"));
    p_file_menu->AppendSeparator();
    p_file_menu->Append(EVENT_FILE_QUIT, _T("E&xit\tAlt-X"), _T(""));
    p_file_menu->AppendSeparator();
    p_file_menu->Append(EVENT_ABOUT, _T("&About"), _T("About"));

    // open/save default directory
    mDir = wxGetHomeDir();

    // edit menu
    wxMenu* p_edit_menu = new wxMenu;
    p_edit_menu->Append(EVENT_UNDO, _T("&Undo Empty\tCtl-z"), _T("Undo last protein manipulation"));
    p_edit_menu->AppendSeparator();
    p_edit_menu->Append(EVENT_BUILD_HYDROGEN, _T("Build &Hydrogens"), _T("Build missing hydrogen atoms"));
    p_edit_menu->Append(EVENT_BLOCK_TERMINALS, _T("&Block Terminals"), _T("Add the NME and ACE blocking group to chain terminii"));
    p_edit_menu->Append(EVENT_CHARGE_TERMINALS, _T("C&harge Terminals"), _T("Modify terminii into charged groups"));
    p_edit_menu->AppendSeparator();
    p_edit_menu->Append(EVENT_ALANINE_PEPTIDE, _T("&Alanine peptide"), _T("Load a alanine 12-mer"));
    p_edit_menu->Append(EVENT_ALPHABET_PROTEIN, _T("A&lphabet protein"), _T("Load the alphabet protein of all 20 amino acids"));

    // create display menu
    wxMenu* p_display_menu = new wxMenu;

    p_display_menu->AppendRadioItem(EVENT_DISPLAY_EVERYTHING, _T("&Everything"));
    p_display_menu->AppendRadioItem(EVENT_DISPLAY_BACKBONE, _T("&Backbone"));
    p_display_menu->AppendRadioItem(EVENT_DISPLAY_CA_TRACE, _T("&Calpha"));
    p_display_menu->AppendRadioItem(EVENT_DISPLAY_CA_SIDECHAIN, _T("&Sidechain/Calpha"));
    p_display_menu->Check(EVENT_DISPLAY_EVERYTHING, true);
    p_display_menu->AppendSeparator();

    p_display_menu->AppendCheckItem(EVENT_DISPLAY_PEPTIDE, _T("&Peptide plane"));
    p_display_menu->Check(EVENT_DISPLAY_PEPTIDE, true);
    p_display_menu->AppendCheckItem(EVENT_DISPLAY_HYDROGEN, _T("&Hydrogen"));
    p_display_menu->Check(EVENT_DISPLAY_HYDROGEN, false);

    p_display_menu->AppendSeparator();

    p_display_menu->AppendCheckItem(EVENT_DISPLAY_HBOND, _T("H-bon&ds"));
    p_display_menu->Check(EVENT_DISPLAY_HBOND, true);
    p_display_menu->AppendCheckItem(EVENT_DISPLAY_WEAK_HBOND, _T("&Weak H-bonds"));
    p_display_menu->Check(EVENT_DISPLAY_WEAK_HBOND, true);
    p_display_menu->AppendCheckItem(EVENT_DISPLAY_STERIC_CLASH, _T("&Steric Clashes"));
    p_display_menu->Check(EVENT_DISPLAY_STERIC_CLASH, true);

    p_display_menu->AppendSeparator();

    p_display_menu->AppendCheckItem(EVENT_DISPLAY_Z_DEPTH, _T("&Z-Depth Light"));
    p_display_menu->Check(EVENT_DISPLAY_Z_DEPTH, true);
    p_display_menu->AppendSeparator();

    p_display_menu->AppendRadioItem(EVENT_DISPLAY_SOLID, _T("B&all && Stick"));
    p_display_menu->AppendRadioItem(EVENT_DISPLAY_LINE, _T("&Line"));

    // create menu bar
    wxMenuBar* p_menu_bar = new wxMenuBar( wxMB_DOCKABLE );

    p_menu_bar->Append(p_file_menu, _T("&File"));
    p_menu_bar->Append(p_edit_menu, _T("&Edit"));
    p_menu_bar->Append(p_display_menu, _T("&Show"));

    SetMenuBar(p_menu_bar);

    make_alphabet_protein(&mProtein);

    CreateStatusBar();
    SetStatusText(_T(mProtein.getName().c_str()));

    // initialize the controller of the protein   
    mpController = new Controller(this, &mProtein);

    // make the view of the protein with attendent widgets
    mpView = new View(this);

    mpProtView = new ProtView(this, &mProtein, mpController);
    mpView->addWidget(mpProtView);

    mpSlabBar = new SlabBar(this, mpProtView, -5, -25, 10, 100);
    mpView->addWidget(mpSlabBar);

    mpRamaPlot = new RamaPlot(this, mpController, 5, -25, 100);
    mpView->addWidget(mpRamaPlot);

    mpChiBox = new ChiBox(this, mpController, mpRamaPlot, 70);
    mpView->addWidget(mpChiBox);

    mpRamaPlotSizer = new RamaPlotSizer(this, mpRamaPlot);
    mpView->addWidget(mpRamaPlotSizer);

    mpSequence = new Sequence(this, mpController, -15, 28, 15);
    mpView->addWidget(mpSequence);

    mpFocusPicker = new FocusPicker(this, mpSequence, mpProtView, 14, 13);
    mpView->addWidget(mpFocusPicker);

    mpScrollbar = new Scrollbar(this, mpSequence, mpProtView, 1, 11);
    mpView->addWidget(mpScrollbar);

    mpLoopSolutions = new LoopSolutions(this, mpController, -25, -25, 10);
    mpView->addWidget(mpLoopSolutions);

    mpPickControl = new PickControl(this, mpProtView, -5, -5, 15);
    mpView->addWidget(mpPickControl);
}



void Frame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close(true);
}



void Frame::OnShow(wxCommandEvent& WXUNUSED(event))
{
    mpView->Refresh();
}



void Frame::loadProtein(std::string fname)
{
  mpController->loadProtein(fname.c_str());
  mpProtView->initializeFocus();
  Refresh();
}



void Frame::OnOpen(wxCommandEvent& WXUNUSED(event))
{
    wxFileDialog dialog(this, 
                        wxT("Choose a PDB file"), 
                        wxEmptyString, 
                        wxEmptyString, 
                        wxT("PDB files (*.pdb)|*.pdb|All files (*.*)|*.*"), 
                        wxOPEN);
    dialog.SetDirectory(mDir);

    if (dialog.ShowModal() == wxID_OK)
    {
        mFname = dialog.GetPath();
        mDir = dialog.GetDirectory();
        loadProtein(mFname.c_str());
        Refresh();
    }
}



void Frame::OnSave(wxCommandEvent& WXUNUSED(event))
{
    wxFileDialog dialog(this,
                        _T("Save as PDB File"),
                        wxEmptyString,
                        _T("peptide.pdb"),
                        _T("PDB files (*.pdb)|*.pdb"),
                        wxSAVE|wxOVERWRITE_PROMPT);

    dialog.SetDirectory(mDir);
    dialog.SetFilterIndex(1);

    if (dialog.ShowModal() == wxID_OK)
    {
        mDir = dialog.GetDirectory();
        mFname = dialog.GetPath();
        mpController->saveProtein(mFname.c_str());
    }
}


void Frame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    std::string about_string =
    "The Ramachandran Plot Explorer 1.0\n"
    "(c) 2006 Bosco Ho, GPL license\n"
    "http://bosco.infogami.com/rama\n"
    "wxWidgets cross-platform GUI library\n"
    "\n"
    "This program is a distillation of my research\n"
    "on the Ramachandran plot and peptide \n"
    "confromations over the last few years. \n"
    "Focusing on the interactive manipulation of\n"
    "a protein structure, it is currently limited\n"
    "to a single chain.\n"
    "\n"
    "For a crash-course in the Ramachandran plot:\n"
    "  http://bosco.infogami.com/ramachandran_plot\n"
    "For details on the backbone steric clash analysis:\n"
    "  Ho et al, Protein Science 12:2508, 2003\n"
    "For details on the proline ring analysis:\n"
    "  Ho et al, Protein Science 14:1011, 2005\n"
    "For details on the 3-point-clamp:\n"
    "  Coutsias et al, J Comput Chem 25:510, 2004\n"
    ;
    
    (void)wxMessageBox(_T(about_string.c_str()),
                       _T("About"), wxICON_INFORMATION);
}



void Frame::OnUndo(wxCommandEvent& WXUNUSED(event))
{
  if (mpController->canUndo())
  {
    mpController->undo();
    Refresh();
  }
}



void Frame::OnSelect(wxCommandEvent& event)
{
  if (event.GetId() == EVENT_CLEAR_SELECTION)
    mpController->resetSelect();
  if (event.GetId() == EVENT_SELECT_ALL)
    mpController->selectAll();
  if (event.GetId() == EVENT_CUT)
  {
    mpController->copySelectionToClipboard();
    mpController->deleteSelection();
  } 
  if (event.GetId() == EVENT_COPY)
    mpController->copySelectionToClipboard();
  if (event.GetId() == EVENT_DELETE)
    mpController->deleteSelection();
  if (event.GetId() == EVENT_INSERT_ALA)
    mpController->insertAlanine();
  if (event.GetId() == EVENT_INSERT)
    mpController->insertClipboardInProtein(mProtein.mInsertRes);
  if (event.GetId() == EVENT_INSERT_RIGHT)
    mpController->insertClipboardInProtein(mProtein.mInsertRes+1);

  if (event.GetId() == EVENT_INSERT_FROM_PDB)
  {
      wxFileDialog dialog(this, wxT("Choose a PDB file"), wxEmptyString, wxEmptyString, 
                          wxT("PDB files (*.pdb)|*.pdb|All files (*.*)|*.*"), wxOPEN);
      dialog.SetDirectory(mDir);

      if (dialog.ShowModal() == wxID_OK)
      {
          mFname = dialog.GetPath();
          mDir = dialog.GetDirectory();
          mpController->insertFromPdb(mFname.c_str());
          Refresh();
      }
  }

  if (event.GetId() == EVENT_COPY_TO_PDB)
  {
      wxFileDialog dialog(this,
                          _T("Save as PDB File"),
                          wxEmptyString,
                          _T("peptide.pdb"),
                          _T("PDB files (*.pdb)|*.pdb"),
                          wxSAVE|wxOVERWRITE_PROMPT);

      dialog.SetDirectory(mDir);
      dialog.SetFilterIndex(1);

      if (dialog.ShowModal() == wxID_OK)
      {
          mDir = dialog.GetDirectory();
          mFname = dialog.GetPath();
          mpController->copyToPdb(mFname.c_str());
      }
  }


  Refresh();
}




void Frame::OnBackbone(wxCommandEvent& WXUNUSED(event))
{
  TorsionDialog dialog(this, mpController);
  dialog.ShowModal();
  Refresh();
}



void Frame::OnBuildHydrogen(wxCommandEvent& WXUNUSED(event))
{
  mpController->buildHydrogen();
  Refresh();
}



void Frame::OnBlockTerminals(wxCommandEvent& WXUNUSED(event))
{
  mpController->blockTerminals();
  Refresh();
}



void Frame::OnChargeTerminals(wxCommandEvent& WXUNUSED(event))
{
  mpController->chargeTerminals();
  Refresh();
}



void Frame::OnAlaninePeptide(wxCommandEvent& WXUNUSED(event))
{
  mpController->makeAlaninePeptide();
  Refresh();
}



void Frame::OnAlphabetProtein(wxCommandEvent& WXUNUSED(event))
{
  mpController->makeAlphabetProtein();
  Refresh();
}



void Frame::OnDisplayDrawStyle(wxCommandEvent& event)
{
    if (event.GetId() == EVENT_DISPLAY_LINE)
      mpProtView->mIsBallStick = false;
    if (event.GetId() == EVENT_DISPLAY_SOLID)
      mpProtView->mIsBallStick = true;
    Refresh();
}



void Frame::OnDisplayShow(wxCommandEvent& event)
{
  mpProtView->mIsBackbone = false;
  mpProtView->mIsCaTrace = false;
  mpProtView->mIsEverything = false;
  mpProtView->mIsCaSidechain = false;

  if (event.GetId() == EVENT_DISPLAY_BACKBONE)
    mpProtView->mIsBackbone = true;
  if (event.GetId() == EVENT_DISPLAY_CA_TRACE)
    mpProtView->mIsCaTrace = true;
  if (event.GetId() == EVENT_DISPLAY_CA_SIDECHAIN)
    mpProtView->mIsCaSidechain = true;
  if (event.GetId() == EVENT_DISPLAY_EVERYTHING)
    mpProtView->mIsEverything = true;
    
  Refresh();  
}



void Frame::OnDisplayOptions(wxCommandEvent& event)
{
  if (event.GetId() == EVENT_DISPLAY_HYDROGEN)
    mpProtView->mIsHydrogen = event.IsChecked();

  if (event.GetId() == EVENT_DISPLAY_PEPTIDE)
    mpProtView->mIsPeptide = event.IsChecked();

  if (event.GetId() == EVENT_DISPLAY_HBOND)
    mpProtView->mIsHbonds = event.IsChecked();
  
  if (event.GetId() == EVENT_DISPLAY_WEAK_HBOND)
     mpProtView->mIsWeakHbonds = event.IsChecked();
  
  if (event.GetId() == EVENT_DISPLAY_STERIC_CLASH)
    mpProtView->mIsStericClash = event.IsChecked();
  
  if (event.GetId() == EVENT_DISPLAY_Z_DEPTH)
    mpProtView->mIsZDepth = event.IsChecked();
    
  Refresh();
}



void Frame::OnMutateRes(wxCommandEvent& event)
{
  switch (event.GetId()) 
  {
    case EVENT_MUTATE_ALA:  mpController->mutate(ALA); break;
    case EVENT_MUTATE_CYS:  mpController->mutate(CYS); break;
    case EVENT_MUTATE_ASP:  mpController->mutate(ASP); break;
    case EVENT_MUTATE_GLU:  mpController->mutate(GLU); break;
    case EVENT_MUTATE_PHE:  mpController->mutate(PHE); break;
    case EVENT_MUTATE_GLY:  mpController->mutate(GLY); break;
    case EVENT_MUTATE_HIS:  mpController->mutate(HIS); break;
    case EVENT_MUTATE_ILE:  mpController->mutate(ILE); break;
    case EVENT_MUTATE_LYS:  mpController->mutate(LYS); break;
    case EVENT_MUTATE_LEU:  mpController->mutate(LEU); break;
    case EVENT_MUTATE_MET:  mpController->mutate(MET); break;
    case EVENT_MUTATE_ASN:  mpController->mutate(ASN); break;
    case EVENT_MUTATE_PRO:  mpController->mutate(PRO); break;
    case EVENT_MUTATE_GLN:  mpController->mutate(GLN); break;
    case EVENT_MUTATE_ARG:  mpController->mutate(ARG); break;
    case EVENT_MUTATE_SER:  mpController->mutate(SER); break;
    case EVENT_MUTATE_THR:  mpController->mutate(THR); break;
    case EVENT_MUTATE_VAL:  mpController->mutate(VAL); break;
    case EVENT_MUTATE_TRP:  mpController->mutate(TRP); break;
    case EVENT_MUTATE_TYR:  mpController->mutate(TYR); break;
 }
 Refresh();
}




///////////////////////////////////////////////////////////////////////////////
////  Entry point of application  
///////////////////////////////////////////////////////////////////////////////

class App: public wxApp
{
public:
  bool OnInit()
  {
    Frame* p_frame = new Frame;
    p_frame->Show(true);
    SetTopWindow(p_frame);
    if (argc > 1)
      p_frame->loadProtein(argv[1]);
    return true;
  }
};

IMPLEMENT_APP(App)

